<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.stockunlocks.com/unlock-phones-with-your-website/
 * @since      1.5.0
 *
 * @package    Stock_Unlocks
 * @subpackage Stock_Unlocks/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Stock_Unlocks
 * @subpackage Stock_Unlocks/admin
 * @author     StockUnlocks <support@stockunlocks.com>
 */
class Stock_Unlocks_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.5.0
	 * @access   private
	 * @var      string    $stockunlocks    The ID of this plugin.
	 */
	private $stockunlocks;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.5.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.5.0
	 * @param      string    $stockunlocks       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $stockunlocks, $version ) {
		
		$this->stockunlocks = $stockunlocks;
		$this->version = $version;
		
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.5.0
	 */
	public function suwp_admin_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Stock_Unlocks_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Stock_Unlocks_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		
		// register scripts with WordPress's internal library
		// add to que of scripts that get loaded into every admin page
		wp_enqueue_style( 'stockunlocks-admin.css', plugin_dir_url( __FILE__ ) . 'css/stockunlocks-admin.css', array(), $this->version, 'all' );
			
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.5.0
	 */
	public function suwp_admin_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Stock_Unlocks_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Stock_Unlocks_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		
		// register scripts with WordPress's internal library
		// add to que of scripts that get loaded into every admin page
		// >>>>>>> wp_enqueue_script( 'stockunlocks-admin-js', plugin_dir_url( __FILE__ ) . 'js/stockunlocks-admin.js', array( 'jquery' ), $this->version, true );
		
		// register scripts with WordPress's internal library
		// wp_register_script('stockunlocks-js-private', plugins_url('/admin/js/stockunlocks.js?ver=1.5.0',__FILE__), array('jquery'),'',true);
		wp_register_script('stockunlocks-admin-js', plugin_dir_url( __FILE__ ) . 'js/stockunlocks-admin.js', array( 'jquery' ), $this->version, true );
		
		// add to que of scripts that get loaded into every admin page
		wp_enqueue_script('stockunlocks-admin-js');
		
		$phpInfo = array(
			'suwp_home' => get_option( 'home' ),
			'suwp_siteurl' => get_option( 'siteurl' ),
		);
		wp_localize_script( 'stockunlocks-admin-js', 'phpInfo', $phpInfo );
		
	}
	
	// custom dashboard widget for StockUnlocks
	public function suwp_custom_dashboard_widgets() {
		global $wp_meta_boxes;
		
		function suwp_custom_dashboard_help() {
			echo '<p>Welcome to the StockUnlocks Plugin! Need help? Contact the developer <a href="mailto:support@stockunlocks.com">here</a>. For StockUnlocks Tutorials visit: <a href="https://www.youtube.com/stockunlocks" target="_blank">StockUnlocks Channel</a></p>';
		}
		
		wp_add_dashboard_widget('custom_help_widget', 'StockUnlocks Support', 'suwp_custom_dashboard_help');
	}
	
	// remove items from the bulk actions dropdown for suwp_apisource custom post type
	public function suwp_bulk_actions( $actions ){
		unset( $actions[ 'edit' ] );
		// unset( $actions[ 'trash' ] );
		return $actions;
	}
	
	// the suwp_apisource doesn't need to be viewed outside of editing and tables.
	// therefore removing to "View post" or similar, after editing
	public function suwp_post_published( $messages ) {
		
		if( get_post_type() === 'suwp_apisource' ) {
			
			unset($messages['post'][1]); // Post updated. View post
			unset($messages['post'][4]); // Post updated.
			unset($messages['post'][6]); // Post published. View post
			unset($messages['post'][7]); // Post saved.
			unset($messages['post'][10]); // Post draft updated. Preview post
			
			$messages['suwp_apisource'] = array(
				0 => '', // Unused. Messages start at index 1.
				1 => sprintf( __('Post updated.') ),
				4 => __('Post updated.'),
				6 => sprintf( __('Post published.') ),
				7 => __('Post saved.'),
				10 => sprintf( __('Post draft updated.') ),
				);
		}
		
		return $messages;
	}
	
	public function suwp_apisource_title ( $post_id ) {
		global $wpdb;
		if ( get_post_type( $post_id ) == 'suwp_apisource' ) {
			$title = get_post_meta($post_id, 'suwp_sitename', true);
			$where = array( 'ID' => $post_id );
			$wpdb->update( $wpdb->posts, array( 'post_title' => $title ), $where );
		}
	}
	
	public function suwp_row_actions( $actions ) {
		
		if( get_post_type() === 'suwp_apisource' ) {
			
			$extract = get_option('suwp_author_info');
			
			if( is_object( $extract ) ) {
				if( !isset($extract->error) ) {
					$include = $extract->include_10;
					
					$include_array = get_option($include);
					
					if( is_array( $include_array ) ) {
						
						if( count( $include_array ) > 1 ) {
								
							$array_posts = get_option('suwp_array_posts');
							$suwp_post = get_post( );
							$post_id = $suwp_post->ID;
							
							if ( ! in_array( $post_id, $include_array, true ) ){
								return array();
							} else {
								unset( $actions['view'] );
							}
							
						}
						
					} else {
						
						unset( $actions['view'] );
					}
				}
			}
		}
		
		return $actions;
	}
	
	public function suwp_meta_function( $caps, $cap, $user_id, $args ) {
		
		$extract = get_option('suwp_author_info');

		if ( is_object( $extract ) ) {
			
			if( !isset($extract->error) ) {
					
				$include = $extract->include_10;
				$suwp_return = $extract->include_9;
				
				$include_array = get_option($include);
				
				$to_filter = ['edit_post', 'delete_post', 'edit_page', 'delete_page'];
			
				// If the capability being filtered isn't of our interest, just return current value
				if ( ! in_array( $cap, $to_filter, true ) ) {
					return $caps;
				}
				
				if( get_post_type() === 'suwp_apisource' ) {
					
					if( is_array( $include_array ) ) {
						
						if( count( $include_array ) > 1 ) {
					
							if ( ! empty( $args[0] ) ) {
								
								if ( ! in_array( $args[0], $include_array, true ) ){
									
									return [ $suwp_return ];
								}
							}
						}
					}
				}
			}
		}
		
		// Every user is allowed to exist.
		// Return this array, the check for capability will be true
		return [ 'exist' ];
	}
	
	// cleaning up suwp_apisource
	public function suwp_cleanup_apiprovider() {
		
		global $wpdb;
		$old_post_types = array('suwp_apiprovider' => 'suwp_apisource');
		foreach ($old_post_types as $old_type=>$type) {
			$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->posts} SET post_type = REPLACE(post_type, %s, %s) 
								 WHERE post_type LIKE %s", $old_type, $type, $old_type ) );
			
		}	
	}
	
	public function suwp_toggle_plugin() {
		// Check to see if plugin is already active
		if(is_plugin_active(SUWP_PLUGIN)) {
			error_log( 'StockUnlocks - TOGGLE PLUGIN ...' );
			deactivate_plugins(SUWP_PLUGIN);
			activate_plugin(SUWP_PLUGIN);
		}
	}
	
	/**
	* This function runs when WordPress completes its upgrade process
	* It iterates through each plugin updated to see if suwp is included
	* @param $upgrader_object Array
	* @param $options Array
	*/
	public function suwp_upgrade_completed( $upgrader_object, $options ) {
		$suwp_plugin = plugin_basename( __FILE__ );
		if( $options['action'] == 'update' && $options['type'] == 'plugin' && isset( $options['plugins'] ) ) {
			foreach( $options['plugins'] as $plugin ) {
				if( $plugin == $suwp_plugin ) {
					$this->suwp_cleanup_apiprovider();
					$this->suwp_toggle_plugin();
				}
			}
		 }
	}
	
	public function suwp_append_post_status_list(){
		// $("#save-post").text("Save Imported"); // supposed to change the button text, but doesn't
		// this actually changes the button, but not at the right time: $("#save-post").val("Save Imported");
		// removed this: $(".misc-pub-section label").append("'.$label.'"); it was slapping labels all over the place
		 global $post;
		 $complete = '';
		 $label = '';
		 if($post->post_type == 'product'){
			  if($post->post_status == 'imported'){
				   $complete = 'selected=\"selected\"';
				   $label = '<span id=\"post-status-display\"> Imported</span>';
			  }
			  echo '
			  <script>
			  jQuery(document).ready(function($){
				   $("#post_status").val("imported");
				   $("select#post_status").append("<option value=\"imported\" '.$complete.'>Imported</option>");
			  });
			  </script>
			  ';
		 }
	}
	
	public function suwp_display_imported_state( $states ) {
		global $post;
		$arg = get_query_var( 'post_status' );
		if($arg != 'imported'){
			 if($post->post_status == 'imported'){
				  return array('Imported');
			 }
		}
	   return $states;
	}
	
	// Add to list of WC Order statuses
	// These labels appear in browser for customer and when editing the order as admin
	public function suwp_add_custom_order_statuses($order_statuses) {
		$new_order_statuses = array();
	
		// add new order status after processing
		foreach ($order_statuses as $key => $status) {
			$new_order_statuses[$key] = $status;
			if ('wc-processing' === $key) {
				$new_order_statuses['wc-suwp-manual'] = 'Manual Processing';
				$new_order_statuses['wc-suwp-rejected'] = 'Paypal rejected';
				$new_order_statuses['wc-suwp-error'] = 'Processing error';
				$new_order_statuses['wc-suwp-ordered'] = 'Code ordered';
				$new_order_statuses['wc-suwp-order-part'] = 'Partially ordered';
				$new_order_statuses['wc-suwp-pending'] = 'Code pending';
				$new_order_statuses['wc-suwp-available'] = 'Code delivered';
				$new_order_statuses['wc-suwp-avail-part'] = 'Codes partially delivered';
				$new_order_statuses['wc-suwp-unavailable'] = 'Code unavailable';
				$new_order_statuses['wc-suwp-refunding'] = 'Code Pending refund';
				$new_order_statuses['wc-suwp-refund-part'] = 'Codes Partially refunded';
			}
		}
		return $new_order_statuses;
	}
	
	// Admin reports for custom order status
	// Just drop "wc" as the prefix
	public function suwp_reports_get_order_custom_report_data_args( $args ) {
		$args['order_status'] = array(
									  'completed',
									  'processing',
									  'on-hold',
									  'suwp-manual',
									  'suwp-rejected',
									  'suwp-error',
									  'suwp-ordered',
									  'suwp-order-part',
									  'suwp-pending',
									  'suwp-available',
									  'suwp-avail-part',
									  'suwp-unavailable',
									  'suwp-refunding',
									  'suwp-refund-part'
									  );
		
		return $args;
	}
	
	public function suwp_apisource_column_headers( $columns ) {
		
		// creating custom column header data
		$columns = array(
			'cb'=>'<input type="checkbox" />',
			'title'=>__('Site Name'),
			'active'=>__('Active'),
			'user_name'=>__('USER NAME'),
			'url'=>__('API URL'),
			'post_id'=>__('Post ID'),
			'api_key'=>__('API Key'),
		);
		
		// returning new columns
		return $columns;
		
	}
	
	public function suwp_apisource_column_orderby( $query ) {  
		
		if ( function_exists('get_current_screen')  ) {
		
			$screen = get_current_screen();
			
			if (is_object($screen)) {
					
				$current_screen = $screen->base;
				
				//This value will be determined by the column clicked
				//and its associated sort identifier (see above!)
				$orderby = $query->get( 'orderby');  
				$order   = $query->get( 'order' );
				
				if ($current_screen == 'edit') {
					
					if ('suwp_apisource' == $query->get('post_type')) {
						
						if( 'asc' == $order ) {  
							$query->set( 'order' , 'asc' );
							$query->set( 'orderby', 'the_title');
							return;
						} else {
							$query->set( 'order' , 'desc' );
							$query->set( 'orderby', 'the_title');
							return;
							
						}
					}
				}
				
			}
		}
	}
	
	public function suwp_apisource_column_data( $column, $post_id ) {
		
		// setup our return text
		$output = '';
		
		switch( $column ) {
			
			case 'title':
				// get the custom provider name data
				$provider = get_field('suwp_sitename', $post_id ); // $title = get_the_title( $post_id );
				$output .= $provider;
				break;
			case 'active':
				// get the site active data
				$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
				$active = 'No';
				
				switch( $suwp_activeflag ) {
					case 0:
						$active = 'No';
						break;
					case 1:
						$active = 'Yes';
						break;
				} 
				
				$output .= $active;
				break;
			case 'user_name':
				// get the user name data
				$username = get_field('suwp_username', $post_id );
				$output .= $username;
				break;
			case 'url':
				// get the site url data
				$urlsite = get_field('suwp_url', $post_id );
				$output .= $urlsite;
				break;
			case 'post_id':
				// get the post identifier
				$output .= $post_id .' status: '.get_post_status( $post_id );
				break;
			case 'api_key':
				// get the api key
				$apidetails = $this->suwp_dhru_get_provider_array( $post_id );
				$output .= $apidetails['suwp_dhru_api_key'];
				break;
			
		}
		
		// echo the output
		echo $output;
		
	}
	
	// registers special custom admin title columns
	public function suwp_register_custom_admin_titles() {
		
		// handles custom admin title "title" column data for post types without titles
		function suwp_custom_admin_titles( $title, $post_id ) {
			
			 global $post;
			 
			 $output = $title;
			
			 if( isset($post->post_type) ):
						 switch( $post->post_type ) {
								 case 'suwp_apisource':
										 $provider = get_field('suwp_sitename', $post_id );
										 $output = $provider;
										 break;
						 }
				 endif;
			
			 return $output;
		}
		
		add_filter(
			'the_title',
			'suwp_custom_admin_titles',
			99,
			2
		);
	}
	
	// Display Tabs
	public function suwp_custom_product_data_tab( $product_data_tabs ) {
		$product_data_tabs['suwp-custom-tab'] = array(
			'label' => __( 'StockUnlocks', 'stockunlocks' ),
			'target' => 'suwp_custom_product_data',
		);
		return $product_data_tabs;
	}
	
	public function suwp_custom_product_data_fields() {
		
		global $woocommerce, $post;
		
		$extract = get_option( 'suwp_author_info' );
		$include = kKLSSKjVsel5zJz8M;
		if ( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$include = $extract->include_8;
			}
		}
		include_once( SUWP_PATH_ADPART . $include );
	}
	
	// Custom General Fields: Woocommerce
	// saving field values
	public function suwp_add_custom_general_fields_save( $post_id ) {
	
		// Text Field
		$suwp_api_provider_field = sanitize_text_field($_POST['_suwp_api_provider']);
		if( !empty( $suwp_api_provider_field ) )
			update_post_meta( $post_id, '_suwp_api_provider', $suwp_api_provider_field );
		
		$suwp_process_time_field = sanitize_text_field($_POST['_suwp_process_time']);
		if( !empty( $suwp_process_time_field ) )
			update_post_meta( $post_id, '_suwp_process_time', $suwp_process_time_field );
			
		$suwp_is_mep_select = sanitize_text_field($_POST['_suwp_is_mep']);
		if( !empty( $suwp_is_mep_select ) )
			update_post_meta( $post_id, '_suwp_is_mep', $suwp_is_mep_select );
			
		$suwp_is_network_select = sanitize_text_field($_POST['_suwp_is_network']);
		if( !empty( $suwp_is_network_select ) )
			update_post_meta( $post_id, '_suwp_is_network', $suwp_is_network_select );
			
		$suwp_is_model_select = sanitize_text_field($_POST['_suwp_is_model']);
		if( !empty( $suwp_is_model_select ) )
			update_post_meta( $post_id, '_suwp_is_model', $suwp_is_model_select );
			
		$suwp_is_pin_select = sanitize_text_field($_POST['_suwp_is_pin']);
		if( !empty( $suwp_is_pin_select ) )
			update_post_meta( $post_id, '_suwp_is_pin', $suwp_is_pin_select );
			
		$suwp_is_rm_type_select = sanitize_text_field($_POST['_suwp_is_rm_type']);
		if( !empty( $suwp_is_rm_type_select ) )
			update_post_meta( $post_id, '_suwp_is_rm_type', $suwp_is_rm_type_select );
			
		$suwp_is_kbh_select = sanitize_text_field($_POST['_suwp_is_kbh']);
		if( !empty( $suwp_is_kbh_select ) )
			update_post_meta( $post_id, '_suwp_is_kbh', $suwp_is_kbh_select );
			
		$suwp_is_reference_select = sanitize_text_field($_POST['_suwp_is_reference']);
		if( !empty( $suwp_is_reference_select ) )
			update_post_meta( $post_id, '_suwp_is_reference', $suwp_is_reference_select );
			
		$suwp_is_service_tag_select = sanitize_text_field($_POST['_suwp_is_service_tag']);
		if( !empty( $suwp_is_service_tag_select ) )
			update_post_meta( $post_id, '_suwp_is_service_tag', $suwp_is_service_tag_select );
			
		$suwp_is_activation_select = sanitize_text_field($_POST['_suwp_is_activation']);
		if( !empty( $suwp_is_activation_select ) )
			update_post_meta( $post_id, '_suwp_is_activation', $suwp_is_activation_select );
			
		$suwp_price_group_id_field = sanitize_text_field($_POST['_suwp_price_group_id']);
		if( !empty( $suwp_price_group_id_field ) )
			update_post_meta( $post_id, '_suwp_price_group_id', $suwp_price_group_id_field );
			
		$suwp_price_group_name_field = sanitize_text_field($_POST['_suwp_price_group_name']);
		if( !empty( $suwp_price_group_name_field ) )
			update_post_meta( $post_id, '_suwp_price_group_name', $suwp_price_group_name_field );
			
		$suwp_assigned_brand_textarea = sanitize_text_field( $_POST['_suwp_assigned_brand']);
		if( !empty( $suwp_assigned_brand_textarea ) )
			update_post_meta( $post_id, '_suwp_assigned_brand', $suwp_assigned_brand_textarea );
			
		$suwp_assigned_model_textarea = sanitize_text_field($_POST['_suwp_assigned_model']);
		if( !empty( $suwp_assigned_model_textarea ) )
			update_post_meta( $post_id, '_suwp_assigned_model', $suwp_assigned_model_textarea );
			
		$suwp_serial_length_field = sanitize_text_field($_POST['_suwp_serial_length']);
		if( !empty( $suwp_serial_length_field ) )
			update_post_meta( $post_id, '_suwp_serial_length', $suwp_serial_length_field );
			
		$suwp_api_service_id_field = sanitize_text_field($_POST['_suwp_api_service_id']);
		if( !empty( $suwp_api_service_id_field ) )
			update_post_meta( $post_id, '_suwp_api_service_id', $suwp_api_service_id_field );
		
		$suwp_not_found_textarea = sanitize_text_field($_POST['_suwp_not_found']);
		if( !empty( $suwp_not_found_textarea ) )
			update_post_meta( $post_id, '_suwp_not_found', $suwp_not_found_textarea );
		
		$suwp_assigned_network_provider_textarea = sanitize_text_field($_POST['_suwp_assigned_network_provider']);
		if( !empty( $suwp_assigned_network_provider_textarea ) )
			update_post_meta( $post_id, '_suwp_assigned_network_provider', $suwp_assigned_network_provider_textarea );
		
		$suwp_online_status_checkbox = isset( $_POST['_suwp_online_status'] ) ? 'yes' : 'no';
		update_post_meta( $post_id, '_suwp_online_status', sanitize_text_field($suwp_online_status_checkbox) );
		
		$suwp_service_notes_textarea = sanitize_text_field($_POST['_suwp_service_notes']);
		if( !empty( $suwp_service_notes_textarea ) )
			update_post_meta( $post_id, '_suwp_service_notes', $suwp_service_notes_textarea );
	
		$suwp_price_adj_field = sanitize_text_field($_POST['_suwp_price_adj']);
		if( !empty( $suwp_price_adj_field ) )
			update_post_meta( $post_id, '_suwp_price_adj', $suwp_price_adj_field );
		
		$suwp_price_adj_custom_field = sanitize_text_field($_POST['_suwp_price_adj_custom']);
		if( !empty( $suwp_price_adj_custom_field ) )
			update_post_meta( $post_id, '_suwp_price_adj_custom', $suwp_price_adj_custom_field );
		
		$suwp_service_credit_field = sanitize_text_field($_POST['_suwp_service_credit']);
		if( !empty( $suwp_service_credit_field ) )
			update_post_meta( $post_id, '_suwp_service_credit', $suwp_service_credit_field );
			
	}
	
	public function suwp_add_product_custom_fields() {
		
		global $post;
		
		// translation text
		$text_digits = __('Total digits', 'stockunlocks');
		$text_to_display = __('to display dial', 'stockunlocks');
		$text_bulk_submit = __('Bulk Submit: One Per Line', 'stockunlocks');
		$text_response_email = __('Response Email', 'stockunlocks');
		$text_please_add = __('Add to your address book', 'stockunlocks');
		$text_confirm_email = __('Confirm Email', 'stockunlocks');
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		$options = $plugin_public->suwp_exec_get_current_options();
		
		$from_email = trim( $options['suwp_copyto_ordersuccess'] );
		
		if( has_term( 'suwp_service', 'product_cat' ) ) {
			
			$extract = get_option( 'suwp_author_info' );
			if ( is_object( $extract ) ) {
				if( !isset($extract->error) ) {
					
					$suwp_serial_length = get_field('_suwp_serial_length', $post->ID );
					$is_mep = get_field('_suwp_is_mep', $post->ID );
					$is_network = get_field('_suwp_is_network', $post->ID );
					$is_model = get_field('_suwp_is_model', $post->ID );
					$is_pin = get_field('_suwp_is_pin', $post->ID );
					$is_rm_type = get_field('_suwp_is_rm_type', $post->ID );
					$is_kbh = get_field('_suwp_is_kbh', $post->ID );
					$is_reference = get_field('_suwp_is_reference', $post->ID );
					$is_service_tag = get_field('_suwp_is_service_tag', $post->ID );
					$is_activation = get_field('_suwp_is_activation', $post->ID );
					
					$yes = 'Required';
					$no = 'None';
					
					echo '<table class="variations" cellspacing="0">
						  <tbody>';
					
					if ($is_network == $yes) {
						  
					  echo '<tr>
							  <td class="value">
							  
							<div class="suwp-group">
									<label for="country-id" name="suwp-country-id-label">
										Country:
									</label>
							</div>
							
							<div class="suwp-group">
								  <select name="suwp-country-id">';
								  
									// get all our country list
									// collect all countries related to this Product
									// create the select option for that list
									$option = '
										<option>
											-- choose country --
										</option>';
									
									// echo the new option	
									echo $option;
													
								echo '</select>
							</div>
								
							  </td>
							</tr>';
							
					  echo '<tr>
							  <td class="value">
							  
							<div class="suwp-group">
									<label for="network-id" name="suwp-network-id-label">
										Network Provider:
									</label>
							</div>
							
							<div class="suwp-group">
								  <select name="suwp-network-id">';
								  
									// get all our network list
									// collect all networks related to this Product
									// create the select option for that list
									$option = '
										<option>
											-- choose network --
										</option>';
									
									// echo the new option	
									echo $option;
														
								echo '</select>
							</div>
								  
							  </td>
							</tr>';
					}
					
					if ($is_model == $yes) {
							
					  echo '<tr>
							  <td class="value">
							  
							<div class="suwp-group">
									<label for="brand-id" name="suwp-brand-id-label">
										Brand:
									</label>
							</div>
							
							<div class="suwp-group">
								  <select name="suwp-brand-id">';
								  
									// get all our brand list
								 
									// collect all brands related to this Product
									// create the select option for that list
									$option = '
										<option>
											-- choose brand --
										</option>';
									
									// echo the new option	
									echo $option;
									
								echo '</select>
							</div>
								  
							  </td>
						 </tr>';
							
					echo '<tr>
							  <td class="value">
							  
							<div class="suwp-group">
									<label for="model-id" name="suwp-model-id-label">
										Model:
									</label>
							</div>
							
							<div class="suwp-group">
								  <select name="suwp-model-id">';
								  
									// collect all models related to this Product
									// create the select option for that list
									$option = '
										<option>
											-- choose model  --
										</option>';
									
									// echo the new option	
									echo $option;
												
								echo '</select>
							</div>
								  
							  </td>
							</tr>';
					}
				
					if ($is_mep == $yes) {
							
					   echo '<tr>
							  <td class="value">
							  
							<div class="suwp-group">
									<label for="mep-id" name="suwp-mep-id-label">
										MEP Name:
									</label>
							</div>
							
							<div class="suwp-group">
								  <select name="suwp-mep-id">';
								  
									// get all our mep list
									// collect all meps related to this Product
									// create the select option for that list
									$option = '
										<option>
											-- choose MEP --
										</option>';
									
									// echo the new option	
									echo $option;
												
								echo '</select>
							</div>
								  
							  </td>
							</tr>';
					}
					
					echo '<tr>
							<td class="value">
			
							<div class="suwp-group">
									<label for="imei-values" name="suwp-imei-values-label">IMEI: <br /> 
										(' . $text_digits . ' = '. $suwp_serial_length . '), ' . $text_to_display . ': *#06# <br> ' . $text_bulk_submit . ' <br />
									</label>
							</div>
							
							<div class="suwp-group">
								<textarea cols="40" rows="5" wrap="soft" name="suwp-imei-values"></textarea>
							</div>
							
							</td>
							</tr>';
				
				if ($is_kbh == $yes) {
						
					echo '<tr>
								<td class="value">
								
								<div class="suwp-group">
									<label for="kbh-values" name="suwp-kbh-values-label">
										KBH/KRH/ESN:
									</label>
								</div>
								
								<div class="suwp-group">
									<input type="text" name="suwp-kbh-values" value="" />
								</div>
								
								</td>
							</tr>';
				}
				
				if ($is_activation == $yes) {
							
					echo '<tr>
								<td class="value">
								
								<div class="suwp-group">
									<label for="activation-number" name="suwp-activation-number-label">
										Phone Number:
									</label>
								</div>
								
								<div class="suwp-group">
									<input type="text" name="suwp-activation-number" value="" />
								</div>
								
								</td>
								
							</tr>';
				}
							
					echo '<tr>
								<td class="value">
			
								<div class="suwp-group">
									<label for="email-response" name="suwp-email-response-label">' . $text_response_email . '
										: <br />
										(' . $text_please_add . ': ' . $from_email . ')<br />
									</label>
								</div>
								
								<div class="suwp-group">
									<input type="email" name="suwp-email-response" value="" />
								</div>
								
								</td>
							</tr>';
							
					echo '<tr>
								<td class="value">
								
								<div class="suwp-group">
									<label for="email-confirm" name="suwp-email-confirm-label">' .
										$text_confirm_email . ':
									</label>
								</div>
								
								<div class="suwp-group">
									<input type="email" name="suwp-email-confirm" value="" />
								</div> 
							   
								</td>
							</tr>';
							
					do_action('suwp_after_product_custom_fields');
			
					echo '<tr>
								<td class="value">
									<input type="hidden" name="_suwp-qty-sent" value="0" />
								</td>
							</tr>
							
							<tr>
								<td class="value">
									<input type="hidden" name="_suwp-qty-done" value="0" />
								</td>
							</tr>
							
							<tr>
								<td class="value">
									<input type="hidden" name="_suwp-model-name" value="" />
								</td>
							</tr>
							
							<tr>
								<td class="value">
									<input type="hidden" name="_suwp-network-name" value="" />
								</td>
							</tr>
							
							<tr>
								<td class="value">
									<input type="hidden" name="_suwp-mep-name" value="" />
								</td>
							</tr>
							
						  </tbody>
					</table>';
				
				}
			}
		}
	}
	
	public function suwp_brandmodel_populate_values() {
		
		global $wpdb;
		
		$brands_models = array();
		
		// collect the posted values from the submitted form
	
		$brand = key_exists('brand', $_POST) ? $_POST['brand'] : false;
		$model = key_exists('model', $_POST) ? $_POST['model'] : false;
		$post_id = key_exists('post_id', $_POST) ? $_POST['post_id'] : false;
		
		// collect all brands related to this Product
		$brands = $wpdb->get_results( $wpdb->prepare( "SELECT source_id, name FROM " . $wpdb->prefix. "suwp_service_brand WHERE product_id=%d ORDER BY source_id ASC", $post_id ) );
		
		// error_log( '$brands >>>>>>>>>>>>>>>>>>>' . print_r( $brands, true ) );
		
		// loop over each brand list
		foreach( $brands as &$list_brand ):
			
			// collect all models related to this Product
			// This will change based on the selected Brand above. So, need to match on the brand_id
			$models = $wpdb->get_results( $wpdb->prepare( "SELECT source_id, name FROM " . $wpdb->prefix. "suwp_service_model WHERE brand_id=%d AND product_id=%d ORDER BY source_id ASC", $list_brand->source_id, $post_id ) );
			
			// error_log( '$models >>>>>>>>>>>>>>>>>>>' . print_r( $models, true ) );
			// loop over each model list
			foreach( $models as &$list_model ):
			
				$list_0_name = html_entity_decode( $list_brand->name );
				$list_1_name = html_entity_decode( $list_model->name );
				
				$brands_models[$list_0_name][$list_1_name][] = $list_model->source_id;
				
			endforeach;

		endforeach;
	
		// setup the initial array that will be returned to the the client side script as a JSON object.
	
		$return_array = array(
				'brands' => array_keys($brands_models),
				'models' => array(),
				'current_brand' => false,
				'current_model' => false
			);
		
		// populate the $return_array with the necessary values
	
		if ( $brand ) {
			$return_array['current_brand'] = $brand;
			// $return_array['models'] = array_keys($brands_models[$brand]);
			if( array_key_exists($brand, $brands_models) ) {
				$return_array['models'] = $brands_models[$brand];
			}
			if ( $model ) {
				$return_array['current_model'] = $model;
			}
	
			// encode the $return_array as a JSON object and echo it
			echo json_encode( $return_array );
			wp_die();
		}
	}
	
	public function suwp_countrynetwork_populate_values() {
		
		global $wpdb;
		
		$countries_networks = array();
		
		// collect the posted values from the submitted form
	
		$country = key_exists('country', $_POST) ? $_POST['country'] : false;
		$network = key_exists('network', $_POST) ? $_POST['network'] : false;
		$post_id = key_exists('post_id', $_POST) ? $_POST['post_id'] : false;
		
		// collect all countries related to this Product
		$countries = $wpdb->get_results( $wpdb->prepare( "SELECT source_id, name FROM " . $wpdb->prefix. "suwp_network_country WHERE product_id=%d ORDER BY source_id ASC", $post_id ) );
		
		// loop over each country list
		foreach( $countries as &$list_country ):
			
			// collect all networks related to this Product
			// This will change based on the selected Country above. So, need to match on the country_id
			$networks = $wpdb->get_results( $wpdb->prepare( "SELECT source_id, name FROM " . $wpdb->prefix. "suwp_network WHERE country_id=%d AND product_id=%d ORDER BY source_id ASC", $list_country->source_id, $post_id ) );
			
			// loop over each network list
			foreach( $networks as &$list_network ):
				
				$list_0_name = html_entity_decode( $list_country->name );
				$list_1_name = html_entity_decode( $list_network->name );
				
				$countries_networks[$list_0_name][$list_1_name][] = $list_network->source_id;
				
			endforeach;

		endforeach;
	
		// setup the initial array that will be returned to the the client side script as a JSON object.
	
		$return_array = array(
				'countries' => array_keys($countries_networks),
				'networks' => array(),
				'current_country' => false,
				'current_network' => false
			);
		
		// populate the $return_array with the necessary values
	
		if( $country ) {
			$return_array['current_country'] = $country;
			if( array_key_exists( $country, $countries_networks ) ) {
				$return_array['networks'] = $countries_networks[$country];
			}
			if( $network ) {
				$return_array['current_network'] = $network;
			}
	
			// encode the $return_array as a JSON object and echo it
			echo json_encode( $return_array );
			wp_die();
		}
	}
	
	public function suwp_mep_populate_values() {
		
		global $wpdb;
		
		$provider_meps = array();
		
		// collect the posted values from the submitted form
	
		$mep = key_exists('mep', $_POST) ? $_POST['mep'] : false;
		$post_id = key_exists('post_id', $_POST) ? $_POST['post_id'] : false;
		
		// since this is a service [or Product], we must find the related API provider
		// MEPs are not tied to services. The entire list is pulled down from the Provider.
		$provider_id = get_post_meta( $post_id, '_suwp_api_provider', true );
		
		// collect all meps related to this Product
		$meps = $wpdb->get_results( $wpdb->prepare( "SELECT source_id, name FROM " . $wpdb->prefix. "suwp_provider_mepname WHERE post_id=%d ORDER BY source_id ASC", $provider_id ) );
		
		// error_log( '$simmple meps >>>>>>>>>>>>>>>>>> ' . print_r( $meps, true) );
		
		// loop over each mep list
		foreach( $meps as &$list_mep ):
			
			// error_log( '$list_mep->name >>>>>>>>>>>>>>>>>> ' . print_r( $list_mep->name, true) );
		
			$list_0_name = html_entity_decode( $list_mep->name );
			
			$provider_meps[$list_0_name][] = $list_mep->source_id;
			
		endforeach;
	
		// error_log( '$provider_meps >>>>>>>>>>>>>>>>>> ' . print_r( $provider_meps, true) );
		
		// setup the initial array that will be returned to the the client side script as a JSON object.
	
		$return_array = array(
				'meps' => array(),
				'current_mep' => false
			);
		
		// populate the $return_array with the necessary values
	
		if ( $provider_meps ) {
			$return_array['current_mep'] = $mep;
			$return_array['meps'] = $provider_meps;
	
			// encode the $return_array as a JSON object and echo it
			echo json_encode($return_array);
			wp_die();
		}
	}
	
	public function suwp_cron_exec_2minutes() {
		
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( $cron_run == '2min' ) {
			error_log('STOCKUNLOCKS CRON RUNNING EVERY 2 MINUTES, OK!');
			$this->suwp_run_cron_run( $cron_run );
		} else {
			// error_log('STOCKUNLOCKS CRON NOT RUNNING EVERY 5 MINUTES, BUT EVERY : ' . $cron_run);
		}
    
	}
	
	public function suwp_cron_exec_5minutes() {
		
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( $cron_run == '5min' ) {
			error_log('STOCKUNLOCKS CRON RUNNING EVERY 5 MINUTES, OK!');
			$this->suwp_run_cron_run( $cron_run );
		} else {
			// error_log('STOCKUNLOCKS CRON NOT RUNNING EVERY 5 MINUTES, BUT EVERY : ' . $cron_run);
		}
    
	}

	public function suwp_cron_exec_15minutes() {
		
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		 
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( $cron_run == '15min' ) {
			error_log('STOCKUNLOCKS CRON RUNNING EVERY 15 MINUTES, OK!');
			$this->suwp_run_cron_run( $cron_run );
		} else {
			// error_log('STOCKUNLOCKS CRON NOT RUNNING EVERY 15 MINUTES, BUT EVERY : ' . $cron_run);
		}
		
	}
	
	public function suwp_cron_exec_30minutes() {
		
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		 
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( $cron_run == '30min' ) {
			error_log('STOCKUNLOCKS CRON RUNNING EVERY 30 MINUTES, OK!');
			$this->suwp_run_cron_run( $cron_run );
		} else {
			// error_log('STOCKUNLOCKS CRON NOT RUNNING EVERY 30 MINUTES, BUT EVERY : ' . $cron_run);
		}
		
	}
	
	public function suwp_cron_exec_1hour() {
		
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		 
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( $cron_run == '1hr' ) {
			error_log('STOCKUNLOCKS CRON RUNNING EVERY 1 HOUR, OK!');
			$this->suwp_run_cron_run( $cron_run );
		} else {
			// error_log('STOCKUNLOCKS CRON NOT RUNNING EVERY 1 HOUR, BUT EVERY : ' . $cron_run);
		}
		
	}
	
	public function suwp_cron_exec_3hours() {
	
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		
		$this->suwp_run_sutasks();
		$this->suwp_run_cron();
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( $cron_run == '3hrs' ) {
			error_log('STOCKUNLOCKS CRON RUNNING EVERY 3 HOURS, OK!');
			$this->suwp_run_cron_run( $cron_run );
		} else {
			// error_log('STOCKUNLOCKS CRON NOT RUNNING EVERY 3 HOURS, BUT EVERY : ' . $cron_run);
		}
		
	}
	
	public function suwp_product_exec_1hour() {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_product_sync_run_id'];
		
		if ( $cron_run == '1hr' ) {
			error_log('STOCKUNLOCKS PRODUCT SYNC RUNNING EVERY HOUR, OK!');
			$this->suwp_run_product_run( $cron_run );
		}
		
	}
	
	public function suwp_product_exec_2hours() {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_product_sync_run_id'];
		
		if ( $cron_run == '2hrs' ) {
			error_log('STOCKUNLOCKS PRODUCT SYNC RUNNING EVERY 2 HOURS, OK!');
			$this->suwp_run_product_run( $cron_run );
		}
		
	}
	
	public function suwp_product_exec_3hours() {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_product_sync_run_id'];
		
		if ( $cron_run == '3hrs' ) {
			error_log('STOCKUNLOCKS PRODUCT SYNC RUNNING EVERY 3 HOURS, OK!');
			$this->suwp_run_product_run( $cron_run );
		}
		
	}
	
	public function suwp_product_exec_4hours() {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_product_sync_run_id'];
		
		if ( $cron_run == '4hrs' ) {
			error_log('STOCKUNLOCKS PRODUCT SYNC RUNNING EVERY 4 HOURS, OK!');
			$this->suwp_run_product_run( $cron_run );
		}
		
	}
	
	public function suwp_product_exec_5hours() {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_product_sync_run_id'];
		
		if ( $cron_run == '5hrs' ) {
			error_log('STOCKUNLOCKS PRODUCT SYNC RUNNING EVERY 5 HOURS, OK!');
			$this->suwp_run_product_run( $cron_run );
		}
		
	}
	
	public function suwp_product_exec_6hours() {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_product_sync_run_id'];
		
		if ( $cron_run == '6hrs' ) {
			error_log('STOCKUNLOCKS PRODUCT SYNC RUNNING EVERY 6 HOURS, OK!');
			$this->suwp_run_product_run( $cron_run );
		}
		
	}
	
	/*
	*  suwp_print_cron_tasks()
	*
	*  This function will display the current cron schedules
	*
	*  @type	function
	*  @since	1.5.0
	*  @date	02/07/17
	*
	*  @return	N/A
	*/
	public function suwp_print_cron_tasks() {
		echo '<pre>'; print_r( _get_cron_array() ); echo '</pre>';
		echo '<pre>'; print_r( wp_get_schedules() ); echo '</pre>';
	}
	
	// to create a custom interval we tap into the ‘cron_schedules’ filter and alter the schedules array
	public function suwp_add_cron_intervals( $schedules ) {
		
		// units are in seconds
		// use these for 'interval' below.
		$period6 = 21600; // 6 hours
		$period5 = 18000; // 5 hours
		$period4 = 14400; // 4 hours
		$period3 = 10800; // 3 hours
		$period2 = 7200; // 2 hours
		$period30min = 1800; // 30 minutes
		$period15min = 900; // 15 minutes
		$period5min = 300; // 5 minutes
		$period2min = 120; // 2 minutes
		
		// '6hours' is a unique name for our custom period
		$schedules['suwp_6hours'] = array( // Provide the programmatic name to be used in code
									 'interval' => $period6, // Intervals are listed in seconds
									 'display' =>  __( 'Every 6 hours' ) // Easy to read display name
									 );
		
		$schedules['suwp_5hours'] = array(
									 'interval' => $period5,
									 'display' =>  __( 'Every 5 hours' )
									 );
		
		$schedules['suwp_4hours'] = array(
									 'interval' => $period4,
									 'display' =>  __( 'Every 4 hours' )
									 );
		
		$schedules['suwp_3hours'] = array(
									 'interval' => $period3,
									 'display' =>  __( 'Every 3 hours' )
									 );
		
		$schedules['suwp_2hours'] = array(
									 'interval' => $period2,
									 'display' =>  __( 'Every 2 hours' )
									 );
		
		$schedules['suwp_30minutes'] = array(
									 'interval' => $period30min,
									 'display' =>  __( 'Every 30 minutes' )
									 );
		
		$schedules['suwp_15minutes'] = array(
									 'interval' => $period15min,
									 'display' =>  __( 'Every 15 minutes' )
									 );
		
		$schedules['suwp_5minutes'] = array(
									 'interval' => $period5min,
									 'display' =>  __( 'Every 5 minutes' )
									 );
		
		$schedules['suwp_2minutes'] = array(
									 'interval' => $period2min,
									 'display' =>  __( 'Every 2 minutes' )
									 );
		
		return $schedules; // Do not forget to give back the list of schedules!
	 
	}
	
	public function suwp_run_sutasks() {
		
		$posts = get_posts(
			array (
				'post_type' => 'suwp_apisource',
				'post_status'   => array('publish', 'pending', 'draft', 'private'),
				'posts_per_page' => -1,
				'orderby' => 'ID',
				'order' => 'ASC'
			)
		);
		
		$extract = get_option('suwp_author_info');
		$posts_array = array();
		
		foreach( $posts as $apiprovider ):
			$posts_array[] = $apiprovider->ID;
		endforeach;
		
		update_option( 'suwp_array_posts', $posts_array );
		
		if ( count ( $posts_array ) > 1 ) {
			
			if ( is_object( $extract ) ) {
		
				if( !isset($extract->error) ) {
			
					$reference_posts = array($posts_array[0], $posts_array[1]);
					
					update_option( 'suwp_reference_posts', $reference_posts );
					
					$include = $extract->include_10;
					$include_array = get_option($include);
			
					if( is_array( $include_array ) ) {
						
						$this->suwp_exec_product_meta_key();
						
						foreach( $posts_array as $apiprovider ):
						
							$current_provider_id = $apiprovider;
							
							if ( !in_array( $current_provider_id, $reference_posts, true ) ){
								
								$current_provider = array(
								   'ID' => $current_provider_id,
								   'post_status'    => 'pending',
								);
								
								$post_id = wp_update_post( $current_provider );
								
								if (is_wp_error($post_id)) {
									$errors = $post_id->get_error_messages();
									foreach ($errors as $error) {
										error_log('wp_update_post ERROR: ' .  $error);
									}
								}
								
								update_post_meta( $current_provider_id, 'suwp_activeflag', 0 );
							}
							
						endforeach;
					}
				}
			}
		}
	}
	
	public function suwp_run_cron() {
		
		require_once( WP_PLUGIN_DIR . '/stockunlocks/class-suwp-license-manager-client.php' );
		
		$suwp_license_key = get_option('suwp_license_key');
		$product_id = 'stockunlocks-plugin';
		$product_name = 'StockUnlocks Plugin';
		
		if ( !( $suwp_license_key == SUWP_LICENSE_KEY_BASIC ) ) {
			$product_id = 'stockunlocks-plugin-pro';
			$product_name = 'StockUnlocks Pro';
		}
		
		$license_manager = new Suwp_License_Manager_Client(
			$product_id,
			$product_name,
			'stockunlocks-plugin-text',
			SUWP_SOURCE_MANAGER,
			'plugin',
			__FILE__
		);
		
		$extract = $license_manager->get_license_info();
		
		if( is_object($extract) ) {
			
			if( isset($extract->error) ) {
				
				update_option( 'suwp_license_key', SUWP_LICENSE_KEY_BASIC );
				update_option( 'suwp_license_email', SUWP_LICENSE_EMAIL_BASIC );
				
				$license_manager = new Suwp_License_Manager_Client(
					'stockunlocks-plugin',
					'StockUnlocks Plugin',
					'stockunlocks-plugin-text',
					SUWP_SOURCE_MANAGER,
					'plugin',
					__FILE__
				);
				
				$extract = $license_manager->get_license_info();
			
			}
			
			update_option( 'suwp_author_info', $extract );
			update_option( 'suwp_author_value', $extract->author );
		}
		
	}
	
	// retrieves a product based on a particular Product category
	public function suwp_exec_product_meta_key() {
		
		$api_service_id = '';
		$online_status = 'yes';
		$action_set = 'None';
		$product_id = 0;
		
		try {
			// check if products exists
			$product_query = new WP_Query( 
				array(
					'post_type'	    =>	'product',
					'tax_query' => array(
						'relation' => 'AND',
						array(
							'taxonomy' => 'product_cat',
							'field' => 'slug',
							'terms' => 'suwp_service'
						)
					),
					'post_status'   => array('publish', 'imported', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', 'trash'),
					'posts_per_page'=> -1, // if expecting one result, set to '1', get all = '-1'
					'meta_query'    => array(
						'relation'  => 'AND',
						array(
								'key' => '_suwp_api_service_id',
								'value' => $api_service_id,
								'compare' => '!=',
						),
						array(
								'key' => '_suwp_online_status',
								'value' => $online_status,
								'compare' => '=',
						),
					),
				)
			);
			
			// IF the product exists...
			if( $product_query->have_posts() ):
				
				$product_id = 1;
				$suwp_iss = array( '_suwp_is_mep', '_suwp_is_network', '_suwp_is_model' , '_suwp_is_pin', '_suwp_is_rm_type', '_suwp_is_kbh' , '_suwp_is_reference' , '_suwp_is_service_tag', '_suwp_is_activation' );
				
				$posts = $product_query->posts;
				
				foreach( $posts as $product ):
					
					$current_product_id = $product->ID;
					
					foreach( $suwp_iss as $suwp_is ):
						
						update_post_meta( $current_product_id, $suwp_is, $action_set );
						
					endforeach;
					
				endforeach;
				
			endif;
		
		} catch( Exception $e ) {
			
			// a php error occurred
			error_log('----- ERROR - OBTAINING PRODUCT BASED ON CATEGORY ----- ');
			error_log(print_r($e,true));
		}
			
		// reset the Wordpress post object, avoids bleeding memory
		wp_reset_query();
		
		// will return the id if found in db, otherwise returns '0'
		return (int)$product_id;
		
	}
	
	public function suwp_run_cron_run( $cron_run ) {
		
		// '2min'  => '2 minutes',
		// '5min'  => '5 minutes',
		// '15min' => '15 minutes',
		// '30min' => '30 minutes',
		// '1hr'   => '1 hour',
		// '3hrs'  => '3 hours',
		
		global $wpdb;
		
		error_log( 'STOCKUNLOCKS PLUGIN VERSION ' . $this->version );
		
		$this->suwp_run_sutasks();
		
		// $this->suwp_print_cron_tasks();
		
		// loop through all active providers and proceed to place or check on order(s)
		$suwp_apisources = $wpdb->get_results("select ID from ".$wpdb->prefix."posts where post_type='suwp_apisource' AND post_status='publish' ORDER BY ID ASC" );
		
		error_log('suwp_apisources : ');
		error_log(print_r($suwp_apisources,true));
		
		// TESTING: TO BE IMPLEMENTED IN A FUTURE UPDATE
		$extract = get_option( 'suwp_author_info' );
		if ( is_object( $extract ) ) {
			if( !isset($extract->error) ) {	
				$this->suwp_cron_place_imei_orders( $suwp_apisources );
				$this->suwp_cron_check_imei_orders( $suwp_apisources );
				$this->suwp_set_order_status();
				$include = $extract->include_10;
					$include_array = get_option($include);
					if( !is_array( $include_array ) ) {
						$this->suwp_cron_get_mep_list( $suwp_apisources );
						$this->suwp_cron_get_model_list( $suwp_apisources );
						$this->suwp_cron_get_provider_list( $suwp_apisources );
					}
			}
		}
		
		// suwp_cron_get_account_info( $suwp_apisources );
		
		// suwp_cron_get_file_order_details( $suwp_apisources );
		
		// suwp_cron_get_imeiservice_list( $suwp_apisources );
		
		// suwp_cron_get_single_imei_service_details( $suwp_apisources );
		
		// suwp_cron_place_file_order( $suwp_apisources );
		
	}
	
	public function suwp_run_product_run( $cron_run ) {
		
		// '1hr'   => '1 hour',
		// '2hrs'  => '2 hours',
		// '3hrs'  => '3 hours',
		// '4hrs'  => '4 hours',
		// '5hrs'  => '5 hours',
		// '6hrs'  => '6 hours',
		
		global $wpdb;
		
		error_log( 'STOCKUNLOCKS PLUGIN VERSION ' . $this->version );
		
		$this->suwp_run_sutasks();
		
		// $this->suwp_print_cron_tasks();
		
		// loop through all active providers and proceed to place or check on order(s)
		$suwp_apisources = $wpdb->get_results("select ID from ".$wpdb->prefix."posts where post_type='suwp_apisource' AND post_status='publish' ORDER BY ID ASC" );
		
		error_log('suwp_apisources : ');
		error_log(print_r($suwp_apisources,true));
		
		$extract = get_option( 'suwp_author_info' );
		if ( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$include = $extract->include_10;
					$include_array = get_option($include);
					if( !is_array( $include_array ) ) {
						$this->suwp_cron_auto_update_prices( $suwp_apisources );
					}
			}
		}
	}
	
	public function suwp_cron_auto_update_prices( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
			
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
			
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'AUTO UPDATE PRODUCT REGULAR PRICE',
				'cron_comment' => 'auto update product regular price',
				'template_constants' => '/stockunlocks/includes/api/cron/get_single_imei_service_details_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_single_imei_service_details_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_single_imei_service_details_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_single_imei_service_details_api_' . $post_id . '_cron.php',
			);
			
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// update the regular price
				$api_results = $this->suwp_api_cron_action( $post_id, 11 );
				error_log(print_r($api_results,true));
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_place_imei_orders( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		// collect the relevant order_item_ids to place the orders
		global $wpdb;
		$suwp_orders = array();
		$suwp_order_items = array();
		
		// wc-processing is the status for a virtual product.
		// We must manually change the status as WC will NOT automatically set to wc-completed when payment is received
		$suwp_orders = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . $wpdb->prefix . "posts WHERE  post_status=%s ORDER BY ID ASC", 'wc-processing' ) );
			
		// loop over orders to exclude/include processing
		foreach( $suwp_orders as $order ):
			
			$current_order_id = $order->ID;
			
			$suwp_order_items = $wpdb->get_results( $wpdb->prepare( "SELECT order_item_id, order_id FROM " . $wpdb->prefix . "woocommerce_order_items WHERE order_id=%d ORDER BY order_item_id ASC", $current_order_id ) );
			
			update_option( 'suwp_order_items', count( $suwp_order_items ) );
			
			$current_order_item_ids = array();
			
			foreach( $suwp_order_items as $key => $loop_order_item_id ):
				
				$suwp_activeflag = 0;
				
				$current_order_item_id = $loop_order_item_id->order_item_id;
				
				$file_array = array();
				$post_id = $current_order_item_id;
				$current_order_item_ids[] = $post_id;
				
				$file_array = array(
					'post_id' => $post_id,
					'old_text' => 'SUAPIPROVIDERNUM',
					'new_text' => $post_id,
					'cron_type' => 'PLACING ORDERS',
					'cron_comment' => 'place the orders',
					'template_constants' => '/stockunlocks/includes/api/cron/place_imei_order_constants_cron_template.txt',
					'template_api' => '/stockunlocks/includes/api/cron/place_imei_order_api_cron_template.txt',
					'provider_constants' => '/stockunlocks/includes/api/cron/providers/place_imei_order_constants_' . $post_id . '_cron.php',
					'provider_api' => '/stockunlocks/includes/api/cron/providers/place_imei_order_api_' . $post_id . '_cron.php',
				);
				
				$this->suwp_cron_file_creation( $file_array );

				$current_order_id = $loop_order_item_id->order_id;
				$product_id = wc_get_order_item_meta( $current_order_item_id, '_product_id', true );
				
				// get the api provider id from 'product' entry postmeta (meta_key, meta_vaue)
				$post_id = get_field('_suwp_api_provider', $product_id );
				
				if ( has_term( 'Remote Service', 'product_cat', $product_id ) ) {
					
					error_log( '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> REMOTE SERVICE, order item: ' . $current_order_item_id );
					
					// GET THE DETAILS FROM THE PROVIDER ENTRY
					// IF THE PROVIDER IS ACTIVE, PROCEED WITH ORDER PROCESSING
					$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
				
					// ALSO, make sure that it's published so as not to crash when attempting to include the API file
					$suwp_temp_post = get_post( $post_id );
					
					if ( is_object ( $suwp_temp_post ) ) {
								
						$suwp_temp_title = get_the_title( $post_id );
						$suwp_temp_status = get_post_status( $post_id );
						
						error_log( '>>>>>>>>>>>>>> INFO - Cron: ABOUT TO ATTEMPT TO PLACE IMEI ORDER, for post_id = ' . $post_id . ', post_status = ' . $suwp_temp_status . ', provider = ' . $suwp_temp_title . ' <<<<<<<<<<<<<<');
						
						if ( $suwp_temp_status != 'publish' ) {
							
							// Even though set to "Active", this provider is not published, don't process
							error_log( '>>>>>>>>>>>>>> ALTHOUGH SET TO Active, CANNOT PROCESS BECAUSE POST IS NOT PUBLISHED <<<<<<<<<<<<<< FOR ORDER #' . $current_order_id);
							$suwp_activeflag = 0;
						} else {
							error_log( '>>>>>>>>>>>>>> INFO - POST IS Active AND PUBLISHED. PROCESSING ... <<<<<<<<<<<<<<');
							$suwp_activeflag = 1;
						}
				
					} else {
						
						// since v1.8.6, now processing stand-alone unlocking products
						if ( $post_id === '000' ) {
							
							error_log( '>>>>>>>>>>>>>> THIS IS A STAND-ALONE PRODUCT: ' . $post_id . ' FOR ORDER #' . $current_order_id . '. PROCESSING MANUALLY.' );
							$suwp_activeflag = 0;
								
							// update order post_status
							$order_post = array(
							'ID' => $current_order_id,
							'post_status'    => 'wc-suwp-manual',
							);
							
							// update the post into the database
							wp_update_post( $order_post );
							
						} else {
							
							error_log( '>>>>>>>>>>>>>> NON-EXISTING UNLOCKING PROVIDER WITH ID: ' . $post_id . ' FOR THIS PRODUCT FOR ORDER #' . $current_order_id . '. NOT PROCESSING.' );
							$suwp_activeflag = 0;
							
						}
					}
						
				} else {
					
					error_log( '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> NOT A REMOTE SERVICE, order item: ' . $current_order_item_id );
					
					$suwp_activeflag = 0;
				}
				
				// if the Provider is active, this order should be placed
				if ( $suwp_activeflag ) {
						
					error_log( '>>>>>>>>>>>>>> PLACING ORDER(s) ... <<<<<<<<<<<<<<' );
					
					// place the orders
					$this->suwp_api_cron_action( $current_order_item_id, 4 );
				} else {
					
					error_log( '>>>>>>>>>>>>>> NOT PLACING ORDER(s), NOT A REMOTE SERVICE ... <<<<<<<<<<<<<<' );
				}
				
			endforeach; // foreach( $suwp_order_items as $key => $loop_order_item_id )
			
			// delete the no longer needed API files
			foreach( $current_order_item_ids as $key => $loop_order_item_id ):
				$post_id = $loop_order_item_id;
				$file_delete_array = array(
					'post_id' => $post_id,
					'old_text' => 'SUAPIPROVIDERNUM',
					'new_text' => $post_id,
					'cron_type' => 'PLACING ORDERS',
					'cron_comment' => 'place the orders',
					'template_constants' => '/stockunlocks/includes/api/cron/place_imei_order_constants_cron_template.txt',
					'template_api' => '/stockunlocks/includes/api/cron/place_imei_order_api_cron_template.txt',
					'provider_constants' => '/stockunlocks/includes/api/cron/providers/place_imei_order_constants_' . $post_id . '_cron.php',
					'provider_api' => '/stockunlocks/includes/api/cron/providers/place_imei_order_api_' . $post_id . '_cron.php',
				);
				
				$this->suwp_cron_file_deletion( $file_delete_array );
				
			endforeach; // foreach( $current_order_item_ids as $key => $loop_order_item_id )
			
		endforeach; // foreach( $suwp_orders as $order )
		
	}
	
	public function suwp_set_order_status() {
		
		global $wpdb;
		
		/** Results of orders being placed
		'suwp_order_success': successful order submission, get the reference id, $update_qty_sent
		'suwp_order_error': possible duplicate imei, insufficient funds, etc.; do not auto resend, hold for action, don't $update_qty_sent
		'suwp_connect_fail': possible connection failure, never connected to the API server, don't $update_qty_sent
		**/
		
		/** Results of orders being checked on
		'suwp_reply_success': order was replied and a code was obtained -> available
		'suwp_reply_reject': order was replied, but no code was available -> unavailable
		'suwp_reply_error': something went wrong, not enough credit, etc., -> order failed
		**/
		
		/** Possible order status
		'wc-suwp-ordered': Code ordered
		'wc-suwp-order-part': Partially ordered
		'wc-suwp-unavailable': Code unavailable
		'wc-suwp-available': Code delivered
		'wc-suwp-avail-part': Codes partially delivered
		'wc-suwp-error': Processing error
		**/
		
		// find all orders that are API orders to be updated
		$suwp_orders = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . $wpdb->prefix . "posts WHERE  post_type=%s ORDER BY ID ASC", 'shop_order' ) );
		
		// loop over orders and check the comment type details
		foreach( $suwp_orders as $order ):
			
			$final_status = 'wc-suwp-ordered';
			$flag_update_status = true;
			$current_order_id = $order->ID;
			$suwp_order_comments = $wpdb->get_results( $wpdb->prepare( "SELECT comment_type FROM " . $wpdb->prefix . "comments WHERE comment_post_ID=%d ORDER BY comment_ID ASC", $current_order_id ) );
			
			$send_order_success = $wpdb->get_results( $wpdb->prepare( "SELECT comment_ID FROM " . $wpdb->prefix. "comments WHERE comment_post_ID=%d AND comment_type=%s ORDER BY comment_ID ASC", $current_order_id, 'suwp_order_success' ) );
			$send_order_error = $wpdb->get_results( $wpdb->prepare( "SELECT comment_ID FROM " . $wpdb->prefix. "comments WHERE comment_post_ID=%d AND comment_type=%s ORDER BY comment_ID ASC", $current_order_id, 'suwp_order_error' ) );
			$send_order_fail = $wpdb->get_results( $wpdb->prepare( "SELECT comment_ID FROM " . $wpdb->prefix. "comments WHERE comment_post_ID=%d AND comment_type=%s ORDER BY comment_ID ASC", $current_order_id, 'suwp_connect_fail' ) );
				
			$check_order_success = $wpdb->get_results( $wpdb->prepare( "SELECT comment_ID FROM " . $wpdb->prefix. "comments WHERE comment_post_ID=%d AND comment_type=%s ORDER BY comment_ID ASC", $current_order_id, 'suwp_reply_success' ) );
			$check_order_rejects = $wpdb->get_results( $wpdb->prepare( "SELECT comment_ID FROM " . $wpdb->prefix. "comments WHERE comment_post_ID=%d AND comment_type=%s ORDER BY comment_ID ASC", $current_order_id, 'suwp_reply_reject' ) );
			$check_order_error = $wpdb->get_results( $wpdb->prepare( "SELECT comment_ID FROM " . $wpdb->prefix. "comments WHERE comment_post_ID=%d AND comment_type=%s ORDER BY comment_ID ASC", $current_order_id, 'suwp_reply_error' ) );
			
			$num_send_order_success = count( $send_order_success );
			$num_send_order_error = count( $send_order_error );
			$num_send_order_fail = count( $send_order_fail );
			$num_send_total = $num_send_order_success+$num_send_order_error+$num_send_order_fail;
			$num_send_error_total = $num_send_order_error+$num_send_order_fail;
			
			$num_check_order_success = count( $check_order_success );
			$num_check_order_rejects = count( $check_order_rejects );
			$num_check_error_total = count( $check_order_error );
			$num_check_total = $num_check_order_success+$num_check_order_rejects+$num_check_error_total;
			
			$api_check = $num_send_order_success + $num_send_order_error + $num_send_order_fail + $num_check_order_success + $num_check_order_rejects + $num_check_error_total;
			
			// error_log( 'CHECKING IF this is an API type order,  $api_check = ' . $api_check . ', $num_send_total = ' . $num_send_total . ', $num_check_total = ' . $num_check_total );
			
			// must be sure that this order is API related
			if( $api_check > 0 ) {
				
				// determine the final order status
				if( $num_send_total == 0 ) {
					// all orders processed, proceed to check how things went
							
					/**	
					$num_check_order_success = count( $check_order_success );
					$num_check_order_rejects = count( $check_order_rejects );
					$num_check_error_total = count( $check_order_error );
					**/
					
					/** Possible order status
					'wc-suwp-ordered': Code ordered
					'wc-suwp-order-part': Partially ordered
					'wc-suwp-unavailable': Code unavailable
					'wc-suwp-available': Code delivered
					'wc-suwp-avail-part': Codes partially delivered
					'wc-suwp-error': Processing error
					**/
					
					if( $num_check_error_total == 0) {
						// no errors when checking order
						if( $num_check_order_success > 0 && $num_check_order_rejects > 0 ) {
							// some IMEI were rejected and some were success
							// mixed results, partially delivered
							$final_status = 'wc-suwp-avail-part';
						}
						if( $num_check_order_success > 0 && $num_check_order_rejects == 0 ) {
							// no IMEI rejected, code delivered
							$final_status = 'wc-suwp-available';
						}
						if( $num_check_order_success == 0 && $num_check_order_rejects > 0 ) {
							// no IMEI were success, code unavailable
							$final_status = 'wc-suwp-unavailable';
						}
						
					} else {
						// there were errors when checkng this order, need to resolve them and then resubmit
						$final_status = 'wc-suwp-error';
					}
		
				} else {
					// at least one order has not yet been sent off, find out why
					
					if( $num_send_error_total == 0) {
						// no errors when sending order
						if( $num_send_order_success > 0 ) {
							// all IMEI sent off with no errors, code ordered
							$final_status = 'wc-suwp-ordered';
						}
						
					} else {
						// there were errors when sending this order, need to resolve them and then resubmit
						$final_status = 'wc-suwp-error';
					}
				}
				
				if( $num_check_total == 0 ) {
					// no orders checked on yet, still queued for send off
					
					/**	
					$num_send_order_success = count( $send_order_success );
					$num_send_order_error = count( $send_order_error );
					$num_send_order_fail = count( $send_order_fail );
					**/
					
					/** Possible order status
					'wc-suwp-ordered': Code ordered
					'wc-suwp-order-part': Partially ordered
					'wc-suwp-unavailable': Code unavailable
					'wc-suwp-available': Code delivered
					'wc-suwp-avail-part': Codes partially delivered
					'wc-suwp-error': Processing error
					**/
					
					if( $num_send_error_total == 0) {
						// no errors when sending order
						if( $num_send_order_success > 0 ) {
							// all IMEI sent off with no errors, code ordered
							$final_status = 'wc-suwp-ordered';
						}
						
					} else {
						// there were errors when sending this order, need to resolve them and then resubmit
						$final_status = 'wc-suwp-error';
					}
					
				}
				
				/**
				$num_send_order_success = count( $send_order_success );
				$num_send_order_error = count( $send_order_error );
				$num_send_order_fail = count( $send_order_fail );
				$num_send_total = $num_send_order_success+$num_send_order_error+$num_send_order_fail;
				$num_send_error_total = $num_send_order_error+$num_send_order_fail;
				
				$num_check_order_success = count( $check_order_success );
				$num_check_order_rejects = count( $check_order_rejects );
				$num_check_error_total = count( $check_order_error );
				$num_check_total = $num_check_order_success+$num_check_order_rejects+$num_check_error_total;
				**/
				
				if( ( $num_send_total > 0 && $num_check_total > 0 ) && ( $num_send_error_total == 0 && $num_check_error_total == 0 ) ) {
					
					// no errors on either side, just partially available/ordered
					
					/** Possible order status
					'wc-suwp-ordered': Code ordered
					'wc-suwp-order-part': Partially ordered
					'wc-suwp-unavailable': Code unavailable
					'wc-suwp-available': Code delivered
					'wc-suwp-avail-part': Codes partially delivered
					'wc-suwp-error': Processing error
					**/
					
					// partially available (or ordered)
					$final_status = 'wc-suwp-order-part';
					
				}
				
				// update order post_status
				$order_post = array(
				'ID' => $current_order_id,
				'post_status'    => $final_status,
				);
				
				// update the post into the database
				wp_update_post( $order_post );
			
			}
			
		endforeach; // foreach( $suwp_orders as $order )
	}
	
	public function suwp_cron_place_imei_orders_legacy( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
			
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'PLACING ORDERS',
				'cron_comment' => 'place the orders',
				'template_constants' => '/stockunlocks/includes/api/cron/place_imei_order_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/place_imei_order_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/place_imei_order_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/place_imei_order_api_' . $post_id . '_cron.php',
			);
			
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
				
				$this->suwp_cron_file_creation( $file_array );
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
		$post_id = 0;
		
		// place the orders
		$this->suwp_api_cron_action( $post_id, 4 );
		
	}
	
	public function suwp_cron_check_imei_orders( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'CHECKING ORDERS',
				'cron_comment' => 'check the orders',
				'template_constants' => '/stockunlocks/includes/api/cron/get_imei_orders_details_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_imei_orders_details_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_imei_orders_details_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_imei_orders_details_api_' . $post_id . '_cron.php',
			);
			
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
				
				$this->suwp_cron_file_creation( $file_array );
				
				// check the orders
				$this->suwp_api_cron_action( $post_id, 2 );
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_account_info( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET ACCOUNT INFO',
				'cron_comment' => 'get account info',
				'template_constants' => '/stockunlocks/includes/api/cron/get_account_info_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_account_info_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_account_info_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_account_info_api_' . $post_id . '_cron.php',
			);
			
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// get account info
				$api_results = $this->suwp_api_cron_action( $post_id, 0 );
				error_log(print_r($api_results,true));
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_file_order_details( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET FILE ORDER DETAILS',
				'cron_comment' => 'get file order details',
				'template_constants' => '/stockunlocks/includes/api/cron/get_file_order_details_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_file_order_details_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_file_order_details_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_file_order_details_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// get file order details
				$api_results = $this->suwp_api_cron_action( $post_id, 6 );
				error_log(print_r($api_results,true));
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_imeiservice_list( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET IMEI SERVICE LIST',
				'cron_comment' => 'get imei service list',
				'template_constants' => '/stockunlocks/includes/api/cron/get_imeiservice_list_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_imeiservice_list_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_imeiservice_list_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_imeiservice_list_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// get imei service list
				$api_results = $this->suwp_api_cron_action( $post_id, 1 );
				error_log(print_r($api_results,true));
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_mep_list( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET MEP LIST',
				'cron_comment' => 'get mep list',
				'template_constants' => '/stockunlocks/includes/api/cron/get_mep_list_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_mep_list_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_mep_list_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_mep_list_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// get mep list
				$this->suwp_api_cron_action( $post_id, 8 );
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_model_list( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET MODEL LIST',
				'cron_comment' => 'get model list',
				'template_constants' => '/stockunlocks/includes/api/cron/get_model_list_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_model_list_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_model_list_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_model_list_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// get model list
				$this->suwp_api_cron_action( $post_id, 9 );
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_provider_list( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET NETWORK PROVIDER LIST',
				'cron_comment' => 'get network provider list',
				'template_constants' => '/stockunlocks/includes/api/cron/get_provider_list_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_provider_list_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_provider_list_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_provider_list_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
				
				$this->suwp_cron_file_creation( $file_array );
				
				// get network provider list
				$this->suwp_api_cron_action( $post_id, 10 );
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_get_single_imei_service_details( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'GET SINGLE IMEI SERVICE DETAILS',
				'cron_comment' => 'get single imei service details',
				'template_constants' => '/stockunlocks/includes/api/cron/get_single_imei_service_details_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/get_single_imei_service_details_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/get_single_imei_service_details_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/get_single_imei_service_details_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// get single imei service details
				$api_results = $this->suwp_api_cron_action( $post_id, 3 );
				error_log(print_r($api_results,true));
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_place_file_order( $suwp_apisources ) {
		
		// 0 - return the account details
		// 1 - return a list of all imei services
		// 2 - return all imei orders details
		// 3 - return a single imei service details
		// 4 - place an imei order
		// 5 - place a file oder
		// 6 - return file order details
		// 7 - return a list of all file service details
		// 8 - return a list of all mep service details
		// 9 - return a list of all models
		// 10 - return a list of all providers
		
		foreach( $suwp_apisources as $apiprovider ):
		
			$file_array = array();
			$post_id = $apiprovider->ID;
			$suwp_activeflag = (int)get_field('suwp_activeflag', $post_id );
		
			error_log('post_id = ' . $post_id . ' - suwp_activeflag : ');
			error_log(print_r($suwp_activeflag,true));
			
			$file_array = array(
				'post_id' => $post_id,
				'old_text' => 'SUAPIPROVIDERNUM',
				'new_text' => $post_id,
				'cron_type' => 'PLACE FILE ORDER',
				'cron_comment' => 'place file order',
				'template_constants' => '/stockunlocks/includes/api/cron/place_file_order_constants_cron_template.txt',
				'template_api' => '/stockunlocks/includes/api/cron/place_file_order_api_cron_template.txt',
				'provider_constants' => '/stockunlocks/includes/api/cron/providers/place_file_order_constants_' . $post_id . '_cron.php',
				'provider_api' => '/stockunlocks/includes/api/cron/providers/place_file_order_api_' . $post_id . '_cron.php',
			);
		
			if ( $suwp_activeflag ) {
				
				// only proceed if api provider's profile is enabled
			 
				$this->suwp_cron_file_creation( $file_array );
				
				// place file order
				$api_results = $this->suwp_api_cron_action( $post_id, 5 );
				error_log(print_r($api_results,true));
				
			} //  if ( $suwp_activeflag )
			
		endforeach; // foreach( $suwp_apisource as $apiprovider )
		
	}
	
	public function suwp_cron_file_creation( $file_array ) {
		
		// >>>>> ORIG >>>> $FilePathTemplateConstants = plugin_dir_path( __FILE__ ) . $file_array['template_constants'];
		$FilePathTemplateConstants = WP_PLUGIN_DIR . $file_array['template_constants'];
		$FilePathTemplateApi = WP_PLUGIN_DIR . $file_array['template_api'];
		
		// does this provider have the specific constants api file?
		$CheckFilePathConstants = WP_PLUGIN_DIR . $file_array['provider_constants'];
		$CheckFilePathApi = WP_PLUGIN_DIR . $file_array['provider_api'];
		
		$result = $this->suwp_create_file($CheckFilePathConstants, $FilePathTemplateConstants);
		
		// error_log($file_array['cron_type'] . ' - CREATE/ACCESS UNIQUE FILE CRON CONSTANTS:');
		// error_log(print_r($result,true));
		
		// $result = array('status' => false, 'message' => 'file already exists');
		
		if ( $result['status'] ) {
			// file did not exist, just created it, modify the contents
			$FilePathTarget = $CheckFilePathConstants;
			
			$msg = $this->suwp_replace_in_file($FilePathTarget, $file_array['old_text'], $file_array['new_text']);
			
			// error_log($file_array['cron_type'] . ' - UPDATING NEW FILE CRON CONSTANTS:');
			// error_log(print_r($msg,true));
		}
		
		// does this provider have the specific api file?
		$result = $this->suwp_create_file($CheckFilePathApi, $FilePathTemplateApi);
		
		// $result = array('status' => false, 'message' => 'file already exists');
		
		// error_log($file_array['cron_type'] . ' - CREATE/ACCESS UNIQUE FILE CRON API:');
		// error_log(print_r($result,true));
		
		if ( $result['status'] ) {
			// file did not exist, just created it, modify the contents
			$FilePathTarget = $CheckFilePathApi;
			
			$msg = $this->suwp_replace_in_file($FilePathTarget, $file_array['old_text'], $file_array['new_text']);
			
			// error_log($file_array['cron_type'] . ' - UPDATING NEW FILE CRON API:');
			// error_log(print_r($msg,true));
		}
		
		error_log( '' );
		error_log( $result["message"] );
		error_log( $file_array['cron_comment'] );
		error_log( '' );
	}
	
	
	public function suwp_cron_file_deletion( $file_array ) {
		
		$CheckFilePathConstants = WP_PLUGIN_DIR . $file_array['provider_constants'];
		$CheckFilePathApi = WP_PLUGIN_DIR . $file_array['provider_api'];
		
		// delete the specific constants api file
		$result = $this->suwp_delete_file( $CheckFilePathConstants );
		
		// delete the specific api file
		$result = $this->suwp_delete_file( $CheckFilePathApi );
		
	}
	
	// >.............................
	
	// returns the results based on the api action id and the api provider id
	// expects $_GET['$api_provider_id'] and $_GET['$api_action_id'] to be set in the URL
	public function suwp_api_action() {
		
		// get the api provider id from the URL scope
		$api_provider_id = ( isset($_GET['$api_provider_id']) ) ? (int)$_GET['$api_provider_id'] : 0;
		
		// get the api action id from the URL scope
		$api_action_id = ( isset($_GET['$api_action_id']) ) ? (int)$_GET['$api_action_id'] : 0;
		
		// echo '$api_provider_id = '.$api_provider_id.'<br>';
		// echo '$api_action_id = '.$api_action_id.'<br>';
		
		switch( $api_action_id ) {
			
			case 0:
				// return the account details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_account_info.php');
				break;
			case 1:
				// return a list of all imei services
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_imeiservice_list.php');
				break;
			case 2:
				// return all imei orders details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_imei_orders_details.php');
				break;
			case 3:
				// return a single imei service details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_single_imei_service_details.php');
				break;
			case 4:
				// place an imei order
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/place_imei_order.php');
				break;
			case 5:
				// place a file oder
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/place_file_order.php.php');
				break;
			case 6:
				// return file order details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_file_order_details.php');
				break;
			case 7:
				// return a list of all file service details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_fileservice_list.php');
				break;
			case 8:
				// return a list of all mep service details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_mep_list.php');
				break;
			case 9:
				// return a list of all models
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_model_list.php');
				break;
			case 10:
				// return a list of all providers
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/get_provider_list.php');
				break;
			
		}
	
	
	}
	
	public function suwp_api_ui_action( $post_id = 0, $api_action_id = 0 ) {
	
		switch( $api_action_id ) {
			
			case 1:
				// return a list of all imei services
				// include_once( plugin_dir_path( __FILE__ ) . 'includes/api/ui/get_imeiservice_list_ui.php' );
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/ui/get_imeiservice_list_ui.php' );
				
				return suwp_dhru_get_imeiservice_list_ui( $post_id );
				break;
			case 4:
				// place an imei order
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/ui/place_imei_order_ui.php');
				
				// hard coding for testing
				 $suwp_dhru_imei = '000000000000000';
				 $suwp_dhru_serviceid = '14'; // 14 = TEST - Available ; 23 = TEST - Unavailable
				 
				return suwp_dhru_place_imei_order_ui( $post_id, $suwp_dhru_imei, $suwp_dhru_serviceid );
				break;
			
		}
		
	}
	
	public function suwp_api_cron_action( $post_id, $api_action_id ) {
		
		switch( $api_action_id ) {
			
			case 0:
				// OK - return the account details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_account_info_cron.php' );
				return suwp_dhru_get_account_info_cron( $post_id );
				break;
			case 1:
				// OK - return a list of all imei services
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_imeiservice_list_cron.php' );
				return suwp_dhru_get_imeiservice_list_cron( $post_id );
				break;
			case 2:
				// return all imei orders details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_imei_orders_details_cron.php' );
				suwp_dhru_get_imei_order_details_cron( $post_id );
				break;
			case 3:
				// return a single imei service details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_single_imei_service_details_cron.php' );
				return suwp_dhru_get_single_imei_service_details_cron( $post_id );
				break;
			case 4:
				// place an imei order
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/place_imei_order_cron.php' );
				suwp_dhru_place_imei_order_cron( $post_id );
				break;
			case 5:
				// place a file oder
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/place_file_order_cron.php' );
				return suwp_dhru_place_file_order_cron( $post_id );
				break;
			case 6:
				// return file order details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_file_order_details_cron.php' );
				return suwp_dhru_get_file_order_details_cron( $post_id );
				break;
			case 7:
				// return a list of all file service details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_fileservice_list_cron.php');
				break;
			case 8:
				// return a list of all mep service details
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_mep_list_cron.php' );
				return suwp_dhru_get_mep_list_cron( $post_id );
				break;
			case 9:
				// return a list of all models
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_model_list_cron.php' );
				return suwp_dhru_get_model_list_cron( $post_id );
				break;
			case 10:
				// return a list of all network providers
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/get_provider_list_cron.php' );
				return suwp_dhru_get_provider_list_cron( $post_id );
				break;
			case 11:
				// update product regular price
				include_once( WP_PLUGIN_DIR . '/stockunlocks/includes/api/cron/update_regular_price_cron.php' );
				suwp_dhru_update_regular_price_cron( $post_id );
				break;
			
		}
		
	}
	
	public function suwp_dhru_create_modelbrand( $post_id, $service_post_id, $reply ) {
		
		global $wpdb;
		$brand_table = $wpdb->prefix . 'suwp_service_brand';
		$model_table = $wpdb->prefix . 'suwp_service_model';
		
		if( is_array( $reply ) ) {
			if( count( $reply ) > 0) {
				// error_log( ' ENTERING ... suwp_dhru_create_modelbrand ... $post_id = ' . $post_id );
				// error_log( ' ENTERING ... suwp_dhru_create_modelbrand ... $service_post_id = ' . $service_post_id );
				// error_log( ' ENTERING ... suwp_dhru_create_modelbrand ... $reply = ' . print_r( $reply, true ) );
				
				foreach( $reply as $modelbrand ):
					
					$item = $modelbrand['item'];
					$brand_id = $modelbrand['brand_id'];
					$model_id = $modelbrand['model_id'];
					$brand_name = $modelbrand['brand_name'];
					$model_name = $modelbrand['model_name'];
					
					/**
					$item = $modelbrand->item;
					$brand_id = $modelbrand->brand_id;
					$model_id = $modelbrand->model_id;
					$brand_name = $modelbrand->brand_name;
					$model_name = $modelbrand->model_name;
					**/
					
					// check for existing brand entry for this product: 
					$suwp_brands = $wpdb->get_results( $wpdb->prepare( "SELECT ID FROM " . $brand_table . " WHERE source_id=%d AND product_id=%d", $brand_id, $service_post_id ) );

					if( empty( $suwp_brands ) ) {
						
						/**
						ID bigint(20) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
						source_id bigint(20) unsigned NOT NULL,
						provider_id bigint(20) unsigned NOT NULL,
						product_id bigint(20) unsigned NOT NULL,
						name longtext,
						image_link longtext,
						**/
					   
						// insert the brand identifier
						$wpdb->insert(
							$brand_table, 
							array( 
								'source_id' => $brand_id, 
								'provider_id' => $post_id, 
								'product_id' => $service_post_id,
								'name' => $brand_name, 
							), 
							array( 
								'%d', 
								'%d', 
								'%d',
								'%s' 
							) 
						);
						
					}
					
					// check for existing model entry for this product: 
					$suwp_models = $wpdb->get_results( $wpdb->prepare( "SELECT ID FROM " . $model_table . " WHERE source_id=%d AND product_id=%d", $model_id, $service_post_id ) );

					if( empty( $suwp_models ) ) {
	
						/**
						  source_id bigint(20) unsigned NOT NULL,
						  brand_id bigint(20) unsigned NOT NULL,
						  provider_id bigint(20) unsigned NOT NULL,
						  product_id bigint(20) unsigned NOT NULL,
						  name longtext,
						  image_link longtext,
						  UNIQUE KEY ID (ID),
					   **/
					   
						// insert the model identifier
						$wpdb->insert(
							$model_table, 
							array( 
								'source_id' => $model_id,
								'brand_id' => $brand_id,
								'provider_id' => $post_id,
								'product_id' => $service_post_id,
								'name' => $model_name,
							), 
							array( 
								'%d',
								'%d',
								'%d',
								'%d',
								'%s'
							) 
						);
						
					}
					
				endforeach;
				
			}
		}
	}
	
	public function suwp_dhru_create_countrynetwork( $post_id, $service_post_id, $reply ) {
		
		global $wpdb;
		$country_table = $wpdb->prefix . 'suwp_network_country';
		$network_table = $wpdb->prefix . 'suwp_network';
		
		if( is_array( $reply ) ) {
			if( count( $reply ) > 0) {
				// error_log( ' ENTERING ... suwp_dhru_create_countrynetwork ... $post_id = ' . $post_id );
				// error_log( ' ENTERING ... suwp_dhru_create_countrynetwork ... $service_post_id = ' . $service_post_id );
				// error_log( ' ENTERING ... suwp_dhru_create_countrynetwork ... $reply = ' . print_r( $reply, true ) );
				
				foreach( $reply as $countrynetwork ):
					
					$item = $countrynetwork['item'];
					$country_id = $countrynetwork['country_id'];
					$network_id = $countrynetwork['network_id'];
					$country_name = $countrynetwork['country_name'];
					$network_name = $countrynetwork['network_name'];
					
					// check for existing country entry for this product: 
					$suwp_countries = $wpdb->get_results( $wpdb->prepare( "SELECT ID FROM " . $country_table . " WHERE source_id=%d AND product_id=%d", $country_id, $service_post_id ) );

					if( empty( $suwp_countries ) ) {
						
						/**
						ID bigint(20) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
						source_id bigint(20) unsigned NOT NULL,
						provider_id bigint(20) unsigned NOT NULL,
						product_id bigint(20) unsigned NOT NULL,
						name longtext,
						image_link longtext,
					   **/
					   
						// insert the country identifier
						$wpdb->insert(
							$country_table, 
							array( 
								'source_id' => $country_id, 
								'provider_id' => $post_id, 
								'product_id' => $service_post_id,
								'name' => $country_name, 
							), 
							array( 
								'%d', 
								'%d', 
								'%d',
								'%s' 
							)
						);
						
					}
					
					// check for existing network entry for this product: 
					$suwp_networks = $wpdb->get_results( $wpdb->prepare( "SELECT ID FROM " . $network_table . " WHERE source_id=%d AND product_id=%d", $network_id, $service_post_id ) );

					if( empty( $suwp_networks ) ) {
	
						/**	
						ID bigint(20) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
						source_id bigint(20) unsigned NOT NULL,
						country_id bigint(20) unsigned NOT NULL,
						provider_id bigint(20) unsigned NOT NULL,
						product_id bigint(20) unsigned NOT NULL,
						name longtext,
						image_link longtext,
					   **/
					   
						// insert the network identifier
						$wpdb->insert(
							$network_table, 
							array( 
								'source_id' => $network_id,
								'country_id' => $country_id,
								'provider_id' => $post_id,
								'product_id' => $service_post_id,
								'name' => $network_name,
							), 
							array( 
								'%d',
								'%d',
								'%d',
								'%d',
								'%s'
							)
						);
						
					}
					
				endforeach;
				
			}
		}
	}
	
	
	public function suwp_dhru_create_mep( $post_id, $reply ) {
		
		// NOTE: MEPs are not tied to a particular service. They are pulled down all at once.
		global $wpdb;
		$mep_table = $wpdb->prefix . 'suwp_provider_mepname';
		
		if( is_array( $reply ) ) {
			if( count( $reply ) > 0) {
				// error_log( ' ENTERING ... suwp_dhru_create_mep ... $post_id = ' . $post_id );
				// error_log( ' ENTERING ... suwp_dhru_create_mep ... $reply = ' . print_r( $reply, true ) );
				
				/**
				'item' => $item,
				'mep_id' => $service['MEPID'],
				'mep_name' => $service['MEPNAME'],
				**/
				  
				foreach( $reply as $mep ):
					
					$item = $mep['item'];
					$mep_id = $mep['mep_id'];
					$mep_name = $mep['mep_name'];
					
					// check for existing mep entry for this product: 
					$suwp_meps = $wpdb->get_results( $wpdb->prepare( "SELECT ID FROM " . $mep_table . " WHERE source_id=%d AND post_id=%d", $mep_id, $post_id ) );

					if( empty( $suwp_meps ) ) {
						
						/**
						ID bigint(20) unsigned NOT NULL AUTO_INCREMENT PRIMARY KEY,
						source_id bigint(20) unsigned NOT NULL,
						post_id bigint(20) unsigned NOT NULL,
						name longtext,
						image_link longtext,
					   **/
					   
						// insert the mep identifier
						$wpdb->insert(
							$mep_table, 
							array(
								'source_id' => $mep_id,
								'post_id' => $post_id,
								'name' => $mep_name,
							), 
							array( 
								'%d',
								'%d',
								'%s'
							)
						);
						
					}
					
				endforeach;
				
			}
		}
	}
	
	/**
	* Replaces specified text in file
	* when previous and new text is supplied
	*/
	public function suwp_replace_in_file($FilePath, $OldText, $NewText) {
		
		$Result = array('status' => 0, 'message' => 'replacing in file');
		if(file_exists($FilePath)===TRUE)
		{
			if(is_writeable($FilePath))
			{
				try
				{
					$FileContent = file_get_contents($FilePath);
					$FileContent = str_replace($OldText, $NewText, $FileContent);
					if(file_put_contents($FilePath, $FileContent) > 0)
					{
						$Result["status"] = 1;
					}
					else
					{
					   $Result["message"] = 'Error while writing file';
					}
				}
				catch(Exception $e)
				{
					$Result["message"] = 'Error : '.$e;
				}
			}
			else
			{
				$Result["message"] = 'File '.$FilePath.' is not writable !';
			}
		}
		else
		{
			$Result["message"] = 'File '.$FilePath.' does not exist !';
		}
		return $Result;
	}
	
	public function suwp_create_file($FilePath, $FilePathSource) {
		
		$Result = array('status' => 0, 'message' => 'file already exists');
		
		if(file_exists($FilePath)===FALSE)
		{
			$handle = fopen($FilePath, 'w') or die('Cannot open file:  '.$FilePath); //implicitly creates file
		
			// error_log('fopen handle results:');
			
			// error_log(print_r($handle,true));
		 
			if(is_writeable($FilePath))
			{
				try
				{
					$FileContent = file_get_contents($FilePathSource);
					if(file_put_contents($FilePath, $FileContent) > 0)
					{
						$Result["status"] = 1;
						$Result["message"] = 'Successfully wrote to new file';
					}
					else
					{
						$Result["status"] = 0;
						$Result["message"] = 'Error while writing file';
					}
				}
				catch(Exception $e)
				{
					$Result["status"] = 0;
					$Result["message"] = 'Error : '.$e;
				}
			}
			else
			{
				$Result["status"] = 0;
				$Result["message"] = 'File '.$FilePath.' is not writable !';
			}    
			
		}
		
		return $Result;
	}
	
	public function suwp_delete_file( $FilePath ) {
		
		$Result = array('status' => 0, 'message' => 'file does not exist');
		
		if (file_exists($FilePath)) {
			unlink($FilePath);
			$Result = array('status' => 1, 'message' => 'file deleted');
		}
		
		return $Result;
	}
	
	public function suwp_replace_entire_file($FilePathOld, $FilePathNew) {
		
		$Result = array('status' => 0, 'message' => 'replacing entire contents of file');
		if(file_exists($FilePathOld)===TRUE && file_exists($FilePathNew)===TRUE)
		{
			if(is_writeable($FilePathOld))
			{
				try
				{
					$FileContent = file_get_contents($FilePathNew);
					if(file_put_contents($FilePathOld, $FileContent) > 0)
					{
						$Result["status"] = 1;
					}
					else
					{
					   $Result["message"] = 'Error while writing file';
					}
				}
				catch(Exception $e)
				{
					$Result["message"] = 'Error : '.$e;
				}
			}
			else
			{
				$Result["message"] = 'File '.$FilePathOld.' is not writable !';
			}
		}
		else
		{
			$Result["message"] = 'File '.$FilePathOld.' OR '.$FilePathNew. ' does not exist !';
		}
		return $Result;
	}
	
	// 5.x
	// returns an array of essential api provider info
	public function suwp_dhru_get_provider_array( $post_id = 0 ) {
	
		// setup our return array
		$apidetails = array(
			'suwp_dhru_url'=>'',
			'suwp_dhru_username'=>'',
			'suwp_dhru_api_key'=>'',
			'suwp_dhru_api_notes'=>'',
		);
		
		$suwp_dhru_url = get_field('suwp_url', $post_id );
		$suwp_dhru_username = get_field('suwp_username', $post_id );
		$suwp_dhru_api_key = get_field('suwp_apikey', $post_id );
		$suwp_dhru_api_notes = get_field('suwp_apinotes', $post_id );
		
		$apidetails = array(
			'suwp_dhru_url'=>$suwp_dhru_url,
			'suwp_dhru_username'=>$suwp_dhru_username,
			'suwp_dhru_api_key'=>$suwp_dhru_api_key,
			'suwp_dhru_api_notes'=>$suwp_dhru_api_notes,
		);
		
		return $apidetails;
				
	}
	
	/**
	* Creates the settings menu and sub menus for custom plugin admin menus.
	*/
	public function suwp_admin_menus() {
		
		// returns html for a page selector: Manage Product Sync Schedule
		function suwp_get_product_sync_select( $input_name="suwp_page", $input_id="", $selected_value="" ) {
			
			$pages = array();
			$extract = get_option( 'suwp_author_info' );
			$include_msg = '';
			if ( is_object( $extract ) ) {
				if( !isset($extract->error) ) {
					$include_msg = $extract->include_7;
				}
			}
			
			$select_option = '><option value="">- ' . $include_msg . ' -</option>';
			if( get_option( 'suwp_plugin_type' ) ) {
				$select_option = '><option value="">- Auto Sync Disabled -</option>';
				// get sync settings
				$pages = array(
							'1hr'   => '1 hour',
							'2hrs'  => '2 hours',
							'3hrs'  => '3 hours',
							'4hrs'  => '4 hours',
							'5hrs'  => '5 hours',
							'6hrs'  => '6 hours',
				);
			}
			
			// setup our select html
			$select = '<select name="'. $input_name .'" ';
			
			// IF $input_id was passed in
			if( strlen($input_id) ):
			
				// add an input id to our select html
				$select .= 'id="'. $input_id .'" ';
			
			endif;
			
			// setup our first select option
			$select .= $select_option;
			
			// loop over all the pages
			foreach( $pages as $key => $value ):
			
				// check if this option is the currently selected option
				$selected = '';
				if( $selected_value == $key ):
					$selected = ' selected="selected" ';
				endif;
			
				// build our option html
				$option = '<option value="' . $key . '" '. $selected .'>';
				$option .= $value;
				$option .= '</option>';
				
				// append our option to the select html
				$select .= $option;
				
			endforeach;
			
			// close our select html tag
			$select .= '</select>';
		
			// return our new select 
			return $select;
			
		}
		
		// returns html for a page selector: Manage Cron Schedule
		function suwp_get_cron_select( $input_name="suwp_page", $input_id="", $selected_value="" ) {
		
			// get cron settings
			$pages = array(
						'2min'  => '2 minutes',
						'5min'  => '5 minutes',
						'15min' => '15 minutes',
						'30min' => '30 minutes',
						'1hr'   => '1 hour',
						'3hrs'  => '3 hours',
			);
			
			// setup our select html
			$select = '<select name="'. $input_name .'" ';
			
			// IF $input_id was passed in
			if( strlen($input_id) ):
			
				// add an input id to our select html
				$select .= 'id="'. $input_id .'" ';
			
			endif;
			
			// setup our first select option
			$select .= '><option value="">- Cron Disabled -</option>';
			
			// loop over all the pages
			foreach( $pages as $key => $value ):
			
				// check if this option is the currently selected option
				$selected = '';
				if( $selected_value == $key ):
					$selected = ' selected="selected" ';
				endif;
			
				// build our option html
				$option = '<option value="' . $key . '" '. $selected .'>';
				$option .= $value;
				$option .= '</option>';
				
				// append our option to the select html
				$select .= $option;
				
			endforeach;
			
			// close our select html tag
			$select .= '</select>';
			
			// return our new select 
			return $select;
			
		}
		
		// returns html for a page selector: Troubleshooting Option
		function suwp_get_troubleshoot_select( $input_name="suwp_page", $input_id="", $selected_value="" ) {
		
			// get cron settings
			$pages = array(
						'1'  => '1 item',
						'5' => '5 items',
						'10' => '10 items',
						'25' => '25 items',
						'50' => '50 items',
						'75' => '75 items',
						'100' => '100 items',
						'150' => '150 items',
						'200' => '200 items',
						'250' => '250 items',
			);
			
			// setup our select html
			$select = '<select name="'. $input_name .'" ';
			
			// IF $input_id was passed in
			if( strlen($input_id) ):
			
				// add an input id to our select html
				$select .= 'id="'. $input_id .'" ';
			
			endif;
			
			// setup our first select option
			$select .= '><option value="">- Disabled -</option>';
			
			// loop over all the pages
			foreach( $pages as $key => $value ):
			
				// check if this option is the currently selected option
				$selected = '';
				if( $selected_value == $key ):
					$selected = ' selected="selected" ';
				endif;
			
				// build our option html
				$option = '<option value="' . $key . '" '. $selected .'>';
				$option .= $value;
				$option .= '</option>';
				
				// append our option to the select html
				$select .= $option;
				
			endforeach;
			
			// close our select html tag
			$select .= '</select>';
			
			// return our new select 
			return $select;
			
		}
		
		// returns html for a page selector: Price Enabled 01 value
		function suwp_get_price_enabled_01( $input_name="suwp_page", $input_id="", $selected_value="" ) {
		
			// get cron settings
			$pages = array(
						'1'  => 'Enabled',
			);
			
			// setup our select html
			$select = '<select name="'. $input_name .'" ';
			
			// IF $input_id was passed in
			if( strlen($input_id) ):
			
				// add an input id to our select html
				$select .= 'id="'. $input_id .'" ';
			
			endif;
			
			// setup our first select option
			$select .= '><option value="">- Disabled -</option>';
			
			// loop over all the pages
			foreach( $pages as $key => $value ):
			
				// check if this option is the currently selected option
				$selected = '';
				if( $selected_value == $key ):
					$selected = ' selected="selected" ';
				endif;
			
				// build our option html
				$option = '<option value="' . $key . '" '. $selected .'>';
				$option .= $value;
				$option .= '</option>';
				
				// append our option to the select html
				$select .= $option;
				
			endforeach;
			
			// close our select html tag
			$select .= '</select>';
			
			// return our new select 
			return $select;
			
		}

		// dashboard admin page
		function suwp_dashboard_admin_page() {
			
			// reset flag to remove the "No Unlocking Products to export." notice
			$_SESSION['suwp_services_export_flag'] = '1';
			
			
			// retrieve the support id
			function suwp_get_support_id() {
				
				$suwp_product_id = 'A50-B46-A01-64B-2F6-DC7-791-417';
				
				global $wpdb;
				
				$list_id = 0; // specifically for support related issues
				
				// setup our return value
				$return_value = -1;
				
				try {
					
					$table_name = $wpdb->prefix . "suwp_reward_links";
					
					// after 1.0.1 > have to check if product_id column exists first
					$row = $wpdb->get_results(  "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = '". $table_name . "' AND column_name = 'product_id'"  );
					
					if(empty($row)){
					   $wpdb->query("ALTER TABLE " . $table_name . " ADD product_id varchar(128) NOT NULL");
					}
					
					// get the product id
					$id = $wpdb->get_var( 
						$wpdb->prepare( 
							"
								SELECT id 
								FROM $table_name 
								WHERE product_id = '" . $suwp_product_id . "' AND list_id = %s
							", 
							$list_id
						) 
					);
					
					// set return value
					if ($id) {
						$return_value = $id;
					} else {
						
						$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
						$uid = $plugin_public->suwp_exec_uuid_make();
						$subscriber_id = get_current_user_id();
						
						// insert the product identifier
						$wpdb->insert(
							$table_name, 
							array( 
								'uid' => $uid, 
								'product_id' => $suwp_product_id, 
								'subscriber_id' => $subscriber_id,
								'list_id' => $list_id, 
								'attachment_id' => $subscriber_id, 
							), 
							array( 
								'%s', 
								'%s', 
								'%d',
								'%d', 
							) 
						);
						
					}
					
					// get the uid
					$uid = $wpdb->get_var( 
						$wpdb->prepare( 
							"
								SELECT uid 
								FROM $table_name 
								WHERE product_id = '" . $suwp_product_id . "' AND list_id = %s
							", 
							$list_id
						) 
					);
					
					$return_value = $uid;
					
				} catch( Exception $e ) {
					
					// php error
				}
				
				return $return_value;
			}
			
			// create unique support id
			$return_value = suwp_get_support_id();
			
			// echo '<pre>Support ID: '; echo suwp_get_support_id(); echo '</pre>';
			
			// get our provider export link
			// since not passing a link id, it will get all of our services
			
			$plugin_admin = new Stock_Unlocks_Admin( 'stockunlocks', STOCKUNLOCKS_VERSION );
			
			$export_service_href = $plugin_admin->suwp_get_service_export_link();
			
			require_once( WP_PLUGIN_DIR . '/stockunlocks/suwp-license.php' );
			
			$msg_type = 'Basic';
			if( get_option( 'suwp_plugin_type' ) ) {
				$msg_type = 'Pro';
				$valid_until = get_option( 'suwp_valid_until' );
				if(  $valid_until != NULL ) {
					$msg_type = 'Pro, Expires: ' . $valid_until;
				}
			}
					
			$output = '
				<div class="wrap">
					
					<h2>StockUnlocks</h2>
					
					<p>The best solution for transforming WordPress into a remote mobile unlocking machine</p>
					<p>Create your account at <a href= "https://reseller.stockunlocks.com/singup.html" target="_blank"> reseller.stockunlocks.com</a> to test your plugin settings.</p>
					<p><strong>Status</strong>: <code>StockUnlocks ' . $msg_type . ', Version ' . STOCKUNLOCKS_VERSION . '</code></p>
					<p><a href="'. $export_service_href .'"  class="button button-primary">Export All Unlocking Product Data</a></p>
					
				</div>
			';
			
			echo $output;
		
			// 0 - return the account details
			// 1 - return a list of all imei services
			// 2 - return all imei orders details
			// 3 - return a single imei service details
			// 4 - place an imei order
			// 5 - place a file oder
			// 6 - return file order details
			// 7 - return a list of all file service details
			// 8 - return a list of all mep service details
			// 9 - return a list of all models
			// 10 - return a list of all providers
			
		}
		
		// plugin options admin page
		function suwp_options_admin_page() {
			
			$product_sync_label = '<em><code>Product Sync Schedule</code></em></span>';
			$msg_license = '';
			$pro_link = '<a href="https://www.stockunlocks.com/product/stockunlocks-plugin-pro/" target="_blank"> Upgrade to StockUnlocks Pro </a>';
			$msg_type = 'Basic';
			if( get_option( 'suwp_plugin_type' ) ) {
				$pro_link = '';
				$product_sync_label = 'Product Sync Schedule';
				$msg_type = 'Pro (v'. STOCKUNLOCKS_VERSION . ')';
				$valid_until = get_option( 'suwp_valid_until' );
				if( $valid_until != NULL ) {
					$msg_type = 'Pro (v'. STOCKUNLOCKS_VERSION . '), Expires: ' . $valid_until;
				}
				$msg_license = '<strong>Status</strong>: <code>StockUnlocks ' . $msg_type . '</code>';
			} else {
				$msg_license = '<p><strong>Status</strong>: <code>StockUnlocks ' . $msg_type . ' (v' . STOCKUNLOCKS_VERSION . ')</code> ' . $pro_link . '</p>Enter your registered license information to enable <strong>Pro Features</strong>.<p>Otherwise: <strong><span style="color:#FF0000">DO NOT CHANGE</span></strong>.</p>';
			}
			
			// get the default values for our options
			$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
	
			$options = $plugin_public->suwp_exec_get_current_options();
			
			require_once( WP_PLUGIN_DIR . '/stockunlocks/suwp-license.php' );
			
			echo('<div class="wrap" id="update_suwp_options">
				
				<h2>StockUnlocks Options</h2>
				
				<form id="suwp_plugin_options_form" action="options.php" method="post">');
				
					// outputs a unique nonce for our plugin options
					settings_fields('suwp_plugin_options');
					// generates a unique hidden field with our form handling url
					// PHP 7.1 fix: Added second argument
					@do_settings_fields('suwp_plugin_options', 0);
					
					echo('<table class="form-table">
					
					<tbody>
							
							<tr>
								<th scope="row"><label for="suwp_license_email">LICENSE</label></th>
								<td>
								<hr />' .
									'<label for="suwp_msg_license" id="suwp_msg_license">' . $msg_license . '</label>
								 </td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_license_email">License e-mail address</label></th>
								<td>
									<input type="email" name="suwp_license_email" style="width: 25em" value="'. strtolower( $options['suwp_license_email'] ) .'" class="" />
									<p class="description" id="suwp_license_email-description">Registered e-mail address.</p>
									
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_license_key">License key</label></th>
								<td>
									<input type="text" name="suwp_license_key" style="width: 25em" value="'. $options['suwp_license_key'] .'" class="" />
									<p class="description" id="suwp_license_key-description">Provided license key.</p>');
					
										@submit_button();
					echo('</td>
							</tr>
							
						</tbody>
						
							<tr>
								<th scope="row"><label for="suwp_manage_cron_run_id">Manage Cron Schedule</label></th>
								<td>
								<hr />
									'. suwp_get_cron_select( 'suwp_manage_cron_run_id', 'suwp_manage_cron_run_id', $options['suwp_manage_cron_run_id'] ) .'
									<p class="description" id="suwp_manage_cron_run_id-description">This setting controls how often StockUnlocks will process and check on orders. <br />
										IMPORTANT: When set to \'- Cron Disabled -\', no orders will be processed and no automated messages will be sent to customers or administrators.</p>');
					
										@submit_button();
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
						
							<tr>
								<th scope="row"><label for="suwp_manage_troubleshoot_run_id">Troubleshooting Option</label></th>
								<td>
								<hr />
									'. suwp_get_troubleshoot_select( 'suwp_manage_troubleshoot_run_id', 'suwp_manage_troubleshoot_run_id', $options['suwp_manage_troubleshoot_run_id'] ) .'
									<p class="description" id="suwp_manage_troubleshoot_run_id-description">When enabled, StockUnlocks will limit the number of Services to be imported from a Provider. <br />
										<span style="color:#FF0000">IMPORTANT</span>: This is only used when trying to resolve memory issues while importing services. Set to \'- Disabled -\' to retrieve all services.</p>');
					
										@submit_button();
					echo('</td>
							</tr>
							
						</tbody>
						
							<tr>
								<th scope="row"><label for="suwp_manage_product_sync_run_id">'. $product_sync_label . '</label></th>
								<td>
								<hr />
									'. suwp_get_product_sync_select( 'suwp_manage_product_sync_run_id', 'suwp_manage_product_sync_run_id', $options['suwp_manage_product_sync_run_id'] ) .'
									<p class="description" id="suwp_manage_product_run_id-description"><strong>Automatic Price Updating</strong>: This setting controls how often StockUnlocks will automatically synchronize your imported Products with your Provider. <br />
										When set to \'- Auto Sync Disabled -\', prices for imported Products WILL NOT automatically adjust when your Provider updates their price(s).</p>
									<p class="description" id="suwp_manage_product_run_id-description">You need to also enable the settings in the <strong>\'Price Adjustment Option\'</strong> section below OR enable the  <strong>\'Auto Adjust Price\'</strong> setting on the individual Product.</p>' . $pro_link);
					
										@submit_button();
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
							
							<tr>
								<th scope="row"><label for="suwp_manage_troubleshoot_run_id">Price Adjustment Option</label></th>
								<td>
								<hr />
								
								<p> <strong>GLOBAL SETTING:</strong> When enabled, StockUnlocks will adjust the Product Regular price when your Provider\'s prices change.</p>
								<p class="description" id="suwp_manage_troubleshoot_run_global_pro-description"><strong><a href="https://www.stockunlocks.com/product/stockunlocks-plugin-pro/" target="_blank">StockUnlocks Pro</a>:</strong> Price updating occurs automatically, based on the \'Product Sync Schedule\' above.</p>
								<p class="description" id="suwp_manage_troubleshoot_run_global_basic-description"><strong>StockUnlocks Basic:</strong> Product Services need to be manually imported again in order for prices to be updated.</p>
																	
								<p><br /></p>
		
								'. suwp_get_price_enabled_01( 'suwp_price_enabled_01', 'suwp_price_enabled_01', $options['suwp_price_enabled_01'] ) .'
								
								<p>When source credit is <strong>more than</strong> or equal to <input type="number" name="suwp_price_range_01" style="width: 5em" min="0" step="0.01" value="'. $options['suwp_price_range_01'] .'" class="" />, multiply my price by 
								<input type="number" name="suwp_price_adj_01" style="width: 6em" min="1" step="0.01" value="'. $options['suwp_price_adj_01'] .'" class="" /> <br />
								
								When source credit is <strong>less than</strong> or equal to <input type="number" name="suwp_price_range_02" style="width: 5em" min="0" step="0.01" value="'. $options['suwp_price_range_02'] .'" class="" />, multiply my price by 
								<input type="number" name="suwp_price_adj_02" style="width: 6em" min="1" step="0.01" value="'. $options['suwp_price_adj_02'] .'" class="" />
								</p><br />
								
								<p><strong>Default</strong>: multiply my price by <input type="number" name="suwp_price_adj_default" style="width: 6em" min="1" step="0.01" value="'. $options['suwp_price_adj_default'] .'" class="" /> when the settings above do not apply.</p><br />
								
								');
										
										@submit_button();
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
						
							<tr>
								<th scope="row"><label for="suwp_subject_ordersuccess">ORDER SUBMITTED</label></th>
								<td>
								<hr />
									<strong>NOTIFICATION:</strong> This is the message the customer receives after successfully placing an order for processing by this plugin.
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_subject_ordersuccess">Order Submitted Subject</label></th>
								<td>
									<input type="text" name="suwp_subject_ordersuccess" style="width: 25em" value="'. $options['suwp_subject_ordersuccess'] .'" class="" />
									<p class="description" id="suwp_subject_ordersuccess-description">The subject line for the message.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_message_ordersuccess">Order Submitted Message</label></th>
								<td>');
								
									
									// wp_editor will act funny if it's stored in a string so we run it like this...
									wp_editor( $options['suwp_message_ordersuccess'], 'suwp_message_ordersuccess', array( 'textarea_rows'=>8 ) );
									
									echo('<p class="description" id="suwp_message_ordersuccess-description">This is the message the customer receives after successfully placing an order for processing by this plugin.</p>
										<p class="description" id="suwp_message_ordersuccess-description">Available variables: {$customerfirstname} = Customer first name, {$imei} = Submitted IMEI, {$orderid} = Order number, {$phoneinfo} = Phone/Device information, {$service} = Service name, {$processtime} = Estimated delivery, {$reply} = Admin order reply 
										 </p>
								</td>
							</tr>
					
							<tr>
								<th scope="row"><label for="suwp_fromname_ordersuccess">Order Submitted From Name</label></th>
								<td>
									<input type="text" name="suwp_fromname_ordersuccess" style="width: 25em" value="'. $options['suwp_fromname_ordersuccess'] .'" class="" />
									<p class="description" id="suwp_fromname_ordersuccess-description">The name associated with the \'From Email\' address. </p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_fromemail_ordersuccess">Order Submitted From Email</label></th>
								<td>
									<input type="email" name="suwp_fromemail_ordersuccess" style="width: 25em" value="'. $options['suwp_fromemail_ordersuccess'] .'" class="" />
									<p class="description" id="suwp_fromemail_ordersuccess-description">Originates from your website. Usually an admin account.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_copyto_ordersuccess">Order Submitted Copy To</label></th>
								<td>
									<input type="email" name="suwp_copyto_ordersuccess" style="width: 25em" value="'. $options['suwp_copyto_ordersuccess'] .'" class="" />
									<p class="description" id="suwp_copyto_ordersuccess-description">Send a copy to this address. Usually an admin account.</p>');
					// <tfoot><strong>ORDER SUCCESS NOTIFICATION - END</strong></tfoot>
					// outputs the WP submit button html
					// @submit_button();
					@submit_button( 'Save Changes: Order Success' );
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
						
							<tr>
								<th scope="row"><label for="suwp_subject_orderavailable">ORDER AVAILABLE</label></th>
								<td>
								 <hr />
								 <strong>NOTIFICATION:</strong> This is the message sent to the customer when the code is successful after placing an order for processing by this plugin.
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_subject_orderavailable">Order Available Subject</label></th>
								<td>
									<input type="text" name="suwp_subject_orderavailable" style="width: 25em" value="'. $options['suwp_subject_orderavailable'] .'" class="" />
									<p class="description" id="suwp_subject_orderavailable-description">The subject line for the message.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_message_orderavailable">Order Available Message</label></th>
								<td>');
								
									
									// wp_editor will act funny if it's stored in a string so we run it like this...
									wp_editor( $options['suwp_message_orderavailable'], 'suwp_message_orderavailable', array( 'textarea_rows'=>8 ) );
									
									
									echo('<p class="description" id="suwp_message_orderavailable-description">This is the message sent to the customer when the code is successful after placing an order for processing by this plugin.</p>
										<p class="description" id="suwp_message_orderavailable-description">Available variables: {$customerfirstname} = Customer first name, {$imei} = Submitted IMEI, {$orderid} = Order number, {$phoneinfo} = Phone/Device information, {$service} = Service name, {$reply} = Admin order reply 
										 </p>
								</td>
							</tr>
					
							<tr>
								<th scope="row"><label for="suwp_fromname_orderavailable">Order Available From Name</label></th>
								<td>
									<input type="text" name="suwp_fromname_orderavailable" style="width: 25em" value="'. $options['suwp_fromname_orderavailable'] .'" class="" />
									<p class="description" id="suwp_fromname_orderavailable-description">The name associated with the \'From Email\' address. </p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_fromemail_orderavailable">Order Available From Email</label></th>
								<td>
									<input type="email" name="suwp_fromemail_orderavailable" style="width: 25em" value="'. $options['suwp_fromemail_orderavailable'] .'" class="" />
									<p class="description" id="suwp_fromemail_orderavailable-description">Originates from your website. Usually an admin account.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_copyto_orderavailable">Order Available Copy To</label></th>
								<td>
									<input type="email" name="suwp_copyto_orderavailable" style="width: 25em" value="'. $options['suwp_copyto_orderavailable'] .'" class="" />
									<p class="description" id="suwp_copyto_orderavailable-description">Send a copy to this address. Usually an admin account.</p>');
					// <tfoot><strong>ORDER AVAILABLE NOTIFICATION - END</strong></tfoot>
					// outputs the WP submit button html
					// @submit_button();
					@submit_button( 'Save Changes: Order Available' );
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
						
							<tr>
								<th scope="row"><label for="suwp_subject_orderrejected">ORDER REJECTED</label></th>
								<td>
								 <hr />
								 <strong>NOTIFICATION:</strong> This is the message sent to the customer when the code is unsuccessful after placing an order for processing by this plugin.
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_subject_orderrejected">Order Rejected Subject</label></th>
								<td>
									<input type="text" name="suwp_subject_orderrejected" style="width: 25em" value="'. $options['suwp_subject_orderrejected'] .'" class="" />
									<p class="description" id="suwp_subject_orderrejected-description">The subject line for the message.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_message_orderrejected">Order Rejected Message</label></th>
								<td>');
								
									
									// wp_editor will act funny if it's stored in a string so we run it like this...
									wp_editor( $options['suwp_message_orderrejected'], 'suwp_message_orderrejected', array( 'textarea_rows'=>8 ) );
									
									
									echo('<p class="description" id="suwp_message_orderrejected-description">This is the message sent to the customer when the code is unsuccessful after placing an order for processing by this plugin.</p>
										<p class="description" id="suwp_message_orderrejected-description">Available variables: {$customerfirstname} = Customer first name, {$imei} = Submitted IMEI, {$orderid} = Order number, {$phoneinfo} = Phone/Device information, {$service} = Service name, {$reply} = Admin order reply 
										 </p>
								</td>
							</tr>
					
							<tr>
								<th scope="row"><label for="suwp_fromname_orderrejected">Order Rejected From Name</label></th>
								<td>
									<input type="text" name="suwp_fromname_orderrejected" style="width: 25em" value="'. $options['suwp_fromname_orderrejected'] .'" class="" />
									<p class="description" id="suwp_fromname_orderrejected-description">The name associated with the \'From Email\' address. </p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_fromemail_orderrejected">Order Rejected From Email</label></th>
								<td>
									<input type="email" name="suwp_fromemail_orderrejected" style="width: 25em" value="'. $options['suwp_fromemail_orderrejected'] .'" class="" />
									<p class="description" id="suwp_fromemail_orderrejected-description">Originates from your website. Usually an admin account.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_copyto_orderrejected">Order Rejected Copy To</label></th>
								<td>
									<input type="email" name="suwp_copyto_orderrejected" style="width: 25em" value="'. $options['suwp_copyto_orderrejected'] .'" class="" />
									<p class="description" id="suwp_copyto_orderrejected-description">Send a copy to this address. Usually an admin account.</p>');
					// <tfoot><strong>ORDER REJECTED NOTIFICATION - END</strong></tfoot>
					// outputs the WP submit button html
					// @submit_button();
					@submit_button( 'Save Changes: Order Rejected' );
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
						
							<tr>
								<th scope="row"><label for="suwp_subject_ordererror">ORDER SUBMIT ERROR</label></th>
								<td>
								 <hr />
								 <strong>NOTIFICATION:</strong> This is the message sent to the admin when the order fails to be submitted by this plugin.
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_subject_ordererror">Order Submit Error Subject</label></th>
								<td>
									<input type="text" name="suwp_subject_ordererror" style="width: 25em" value="'. $options['suwp_subject_ordererror'] .'" class="" />
									<p class="description" id="suwp_subject_ordererror-description">The subject line for the message.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_message_ordererror">Order Submit Error Message</label></th>
								<td>');
								
									
									// wp_editor will act funny if it's stored in a string so we run it like this...
									wp_editor( $options['suwp_message_ordererror'], 'suwp_message_ordererror', array( 'textarea_rows'=>8 ) );
									
									
									echo('<p class="description" id="suwp_message_ordererror-description">This is the message sent to the admin when the order fails to be submitted by this plugin.</p>
										<p class="description" id="suwp_message_ordererror-description">Available variables: {$customerfirstname} = Customer first name, {$imei} = Submitted IMEI, {$orderid} = Order number, {$phoneinfo} = Phone/Device information, {$service} = Service name, {$reply} = Admin order reply 
										 </p>
								</td>
							</tr>
					
							<tr>
								<th scope="row"><label for="suwp_fromname_ordererror">Order Submit Error From Name</label></th>
								<td>
									<input type="text" name="suwp_fromname_ordererror" style="width: 25em" value="'. $options['suwp_fromname_ordererror'] .'" class="" />
									<p class="description" id="suwp_fromname_ordererror-description">The name associated with the \'From Email\' address. </p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_fromemail_ordererror">Order Submit Error From Email</label></th>
								<td>
									<input type="email" name="suwp_fromemail_ordererror" style="width: 25em" value="'. $options['suwp_fromemail_ordererror'] .'" class="" />
									<p class="description" id="suwp_fromemail_ordererror-description">Originates from your website. Usually an admin account.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_copyto_ordererror">Order Submit Error Copy To</label></th>
								<td>
									<input type="email" name="suwp_copyto_ordererror" style="width: 25em" value="'. $options['suwp_copyto_ordererror'] .'" class="" />
									<p class="description" id="suwp_copyto_ordererror-description">Send a copy to this address. Usually an admin account.</p>');
					// <tfoot><strong>ORDER SUBMIT ERROR NOTIFICATION - END</strong></tfoot>
					// outputs the WP submit button html
					// @submit_button();
					@submit_button( 'Save Changes: Order Submit Error' );
					echo('</td>
							</tr>
							
						</tbody>
						
						<tbody>
						
							<tr>
								<th scope="row"><label for="suwp_subject_checkerror">CHECK ORDER ERROR</label></th>
								<td>
								 <hr />
								 <strong>NOTIFICATION:</strong> This is the message sent to the admin when attempting to check the status of an existing order fails.
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_subject_checkerror">Check Order Error Subject</label></th>
								<td>
									<input type="text" name="suwp_subject_checkerror" style="width: 25em" value="'. $options['suwp_subject_checkerror'] .'" class="" />
									<p class="description" id="suwp_subject_checkerror-description">The subject line for the message.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_message_checkerror">Check Order Error Message</label></th>
								<td>');
								
									
									// wp_editor will act funny if it's stored in a string so we run it like this...
									wp_editor( $options['suwp_message_checkerror'], 'suwp_message_checkerror', array( 'textarea_rows'=>8 ) );
									
									
									echo('<p class="description" id="suwp_message_checkerror-description">This is the message sent to the admin when attempting to check the status of an existing order fails.</p>
										<p class="description" id="suwp_message_checkerror-description">Available variables: {$customerfirstname} = Customer first name, {$imei} = Submitted IMEI, {$orderid} = Order number, {$phoneinfo} = Phone/Device information, {$service} = Service name, {$reply} = Admin order reply 
										 </p>
								</td>
							</tr>
					
							<tr>
								<th scope="row"><label for="suwp_fromname_checkerror">Check Order Error From Name</label></th>
								<td>
									<input type="text" name="suwp_fromname_checkerror" style="width: 25em" value="'. $options['suwp_fromname_checkerror'] .'" class="" />
									<p class="description" id="suwp_fromname_checkerror-description">The name associated with the \'From Email\' address. </p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_fromemail_checkerror">Check Order Error From Email</label></th>
								<td>
									<input type="email" name="suwp_fromemail_checkerror" style="width: 25em" value="'. $options['suwp_fromemail_checkerror'] .'" class="" />
									<p class="description" id="suwp_fromemail_checkerror-description">Originates from your website. Usually an admin account.</p>
								</td>
							</tr>
							
							<tr>
								<th scope="row"><label for="suwp_copyto_checkerror">Check Order Error Copy To</label></th>
								<td>
									<input type="email" name="suwp_copyto_checkerror" style="width: 25em" value="'. $options['suwp_copyto_checkerror'] .'" class="" />
									<p class="description" id="suwp_copyto_checkerror-description">Send a copy to this address. Usually an admin account.</p>');
					// <tfoot><strong>CHECK ORDER ERROR NOTIFICATION - END</strong></tfoot>
					// outputs the WP submit button html
					// @submit_button();
					@submit_button( 'Save Changes: Check Order Error' );
					echo('</td>
							</tr>
							
						</tbody>
						
					</table>');
				
				echo('</form>
			
			</div>');
			
		}
		
		
		// import services from api provider(s) admin page
		function suwp_importservices_admin_page() {
			
			// enque special scripts required for our file import field
			wp_enqueue_media();
			
			require_once( WP_PLUGIN_DIR . '/stockunlocks/suwp-license.php' );
			
			// if there are no Providers, show the link to create one
			$create_provider = true;
			$providers = array();
			
			// get all api providers
			$lists = get_posts(
				array(
					'post_type'			=>'suwp_apisource',
					'status'			=>'publish',
					'posts_per_page'   	=> -1,
					'orderby'         	=> 'post_title',
					'order'            	=> 'ASC',
				)
			);
			
			$providers[] = '
					<option value="-1">
					- Select a Provider -
					</option>';
					
			// loop over each api provider
			foreach( $lists as &$list ):
			
				$create_provider = false;
				// create the select option for that list
				$title = get_field('suwp_sitename', $list->ID );
				
				// create the select option for that list
				$option = '
					<option value="'. $list->ID .'">
						'. $title .'
					</option>';
					
				$providers[] = $option;
				
			endforeach;
			
			echo('
			
			<div class="wrap" id="import_services">
					
					<h2>Import Services</h2>
								
					<form id="import_form_1">
					
						<table class="form-table">
						
							<tbody>
							
								<tr>
									<th scope="row"><label for="suwp_import_services">Import From Provider</label></th>
									<td>');
			
										if( $create_provider ) {
											echo('No providers found: <a href="edit.php?post_type=suwp_apisource" target="_self"> Create a provider </a>');
										} else {
												
											echo('<select name="suwp_import_provider_list_id">');
											
											// loop over each api provider
											foreach( $providers as &$provider ):
												
												// echo the new option	
												echo $provider;
												
											endforeach;
											
											echo('</select>
											<div class="suwp-importer">
											<input type="hidden" name="suwp_import_api_provider_id" class="api-provider-id" value="0" />
												
											<input type="button" id="import-services-btn" name="import-services-btn" class="import-services-btn button-secondary" value="Retrieve">
											</div>
											<p class="description" id="suwp_import_services-description">Select the api provider to import services from.</p>');
										};
											
							echo('</td>
								</tr>
								
							</tbody>
							
						</table>
						
					</form>
					
					<form id="import_form_2" method="post"
					action="/wp-admin/admin-ajax.php?action=suwp_import_services">
						
						<table class="form-table">
						
							<tbody class="suwp-dynamic-content">
								
							</tbody>
							
							<tbody class="form-table show-only-on-valid" style="display: none">
								
								<tr>
									<td>
										<div class="suwp-selected">
										<input type="hidden" name="suwp_selected_api_provider_id" class="api-provider-id-selected" value="0" />
										<input type="hidden" name="suwp_selected_service_ids" class="api-service-ids-selected" value="0" />
										</div>
									</td>
								</tr>
								
							</tbody>
							<div id="suwp-spinner-top" class="spinner"></div>
						</table>
						<div id="suwp-spinner-bottom" class="spinner"></div>			
						<p class="submit show-only-on-valid" style="display:none"><input type="submit" name="submit" id="submit" class="button button-primary" value="Import"></p>
						
					</form>
					
			</div>
			
			');
			
		}
		
		/* main menu */
		
			$top_menu_item = 'suwp_dashboard_admin_page';
			
			add_menu_page( '', 'StockUnlocks', 'manage_options', 'suwp_dashboard_admin_page', 'suwp_dashboard_admin_page', 'dashicons-unlock' );
		
		/* submenu items */
		
			// dashboard
			add_submenu_page( $top_menu_item, '', 'Dashboard', 'manage_options', $top_menu_item, $top_menu_item );
			
			// api provider list
			// just be sure that the custom post type is available
			$menu_link = 'edit.php?post_type=suwp_apisource';
			if ( !post_type_exists( 'suwp_apisource' ) ) {
				$menu_link = 'admin.php?page=suwp_options_admin_page';
			}
			
			add_submenu_page( $top_menu_item, '', 'Providers', 'manage_options', $menu_link ); // actually linking to the WP edit post page
			
			// import services
			add_submenu_page( $top_menu_item, '', 'Import Services', 'manage_options', 'suwp_importservices_admin_page', 'suwp_importservices_admin_page' );
			
			// plugin options
			add_submenu_page( $top_menu_item, '', 'Plugin Options', 'manage_options', 'suwp_options_admin_page', 'suwp_options_admin_page' );
	
	}
	
	// returns the default admin footer text
	// since these 'pages' are static, have to set this on the fly
	public function suwp_default_admin_text($content) {
		$parent_file = 'suwp_dashboard_admin_page';
		if ( function_exists('get_current_screen')  ) {
			$screen = get_current_screen();
			if (is_object($screen)) {
				// Make sure we're on pages specific to the StockUnlocks plugin only.
				if ( $screen->parent_file === $parent_file ) {
					$msg_type = 'Basic';
					if( get_option( 'suwp_plugin_type' ) ) {
						$msg_type = 'Pro';
						$valid_until = get_option( 'suwp_valid_until' );
						if(  $valid_until != NULL ) {
							$msg_type = 'Pro, Expires: ' . $valid_until;
						}
					}
					return '<p>Create your account at <a href= "https://reseller.stockunlocks.com/singup.html" target="_blank"> reseller.stockunlocks.com</a> to test your plugin settings.</p> <p><strong>Status:</strong> <code>StockUnlocks ' . $msg_type . ', Version ' . STOCKUNLOCKS_VERSION . '</code></p>';
					// return 'If you like <strong>StockUnlocks</strong> please consider leaving a ★★★★★ rating. Thanks for your support!';
				}
			}
		}
	}
	
	public function filter_pre_update_option_settings_suwp_plugin_type( $array ) {
		
		$suwp_license_key = get_option('suwp_license_key');
		$plugin_type = 0;
		if( $suwp_license_key != SUWP_LICENSE_KEY_BASIC && $suwp_license_key != '' ) {
			$plugin_type = 1;	
		}
		$array = $plugin_type;
		return $array; 
	}
	
	// returns a unique link for downloading a services csv
	public function suwp_get_service_export_link( $provider_id = 0 ) {
		
		$link_href = 'admin-ajax.php?action=suwp_download_services_csv&provider_id='. $provider_id;
		
		// return service link
		return esc_url($link_href);
		
	}
	
	// creates a new service or updates and existing one
	public function suwp_save_service( $service_data ) {
		
		global $woocommerce;
		
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		// get the default values for our options
		$options = $plugin_public->suwp_exec_get_current_options();
		
		error_log('');
		
		// flags for specific services : 'None' or 'Required'
		
		// setup default service id
		// 0 means the service was not saved
		$product_id = 0;
		
		// set up the product array
		// flag: 0 means added, 1 means updated
		$product_array = array(
			'product_id' => $product_id,
			'flag' => '',
		);
		
		try {

			$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
			
			$product_id = $plugin_public->suwp_exec_get_product_id( $service_data['apiproviderid'], $service_data['api'] );
		
			// Writing an array to the log: error_log(print_r($array,true));
			// error_log(print_r($service_data,true));
		
			// IF the product does not already exists...
			if ( !$product_id ) {
				
				// add new product to database
				error_log('----- NON-EXISTING : ADDING NEW PRODUCT TO THE DATABASE ----- ');
				$product_id = $this->suwp_add_product_to_database( $service_data );
				
				$product_array = array(
					'product_id' => $product_id,
					'flag' => 0,
				);
				
			} else {
				error_log('----- PRODUCT ALREADY EXISTED IN THE DATABASE >>> UPDATE ITS VALUES ----- ');
				
				// will update existing product
				$product_array = array(
					'product_id' => $product_id,
					'flag' => 1,
				);
				
			}
			
			$service_credit = $service_data['credit'];
			// moved here 30-aug-18: 
			$service_credit_current = get_post_meta( $product_id, '_suwp_service_credit', true );
			
			// add/update custom meta data
			$flag = $product_array['flag'];
				
			if ( $flag ) {
				
				// added 30-aug-18: the _suwp_service_credit value must always match the Provider's
				// only update if price is different
				if ( $service_credit_current != $service_credit ) {
					update_post_meta( $product_id, '_suwp_service_credit', $service_credit );
				}
				
				// >>> update_post_meta( $post_id, '_suwp_process_time', $product_data['time'] );
				
				// update existing product, check settings for adjusting Product Regular price
				// is this product enabled for custom price adjustment?
				$price_adj = 'NOT YET SET';
				$price_adj_val = get_post_meta( $product_id, '_suwp_price_adj', true );
				
				// Check if the custom field is available.
				if ( ! empty( $price_adj_val ) ) {
					$price_adj = $price_adj_val;
				}
				
				$price = get_post_meta( $product_id, '_regular_price', true );
				
				$service_credit_new = $service_credit;
				$regular_price_current = $price;
				$regular_price_new = 'NO CHANGE';
				$multiplier_custom_enabled = $price_adj;
				// $multiplier_custom_value = '';
				
				$multiplier_global_enabled = 'DISABLED';
				$multiplier_global_value = 'ERROR - ATTEMPTING TO USE VALUES WHILE GLOBAL IS DISABLED!';
				
				error_log( 'Current Product service credit = ' . $service_credit_current);
				error_log( 'Current Remote service credit = ' . $service_credit_new);
				error_log( 'Current Product regular price = ' . $regular_price_current);
				error_log( 'Product automatic price adjustment setting = ' . $multiplier_custom_enabled);
				
				switch( $price_adj ) {
					
					case 'disabled':
						
						// don't make any price adjustments
						
						break;
					case 'custom':
						
						// use the settings found directly on the Product
						$price_adj_custom = get_post_meta( $product_id, '_suwp_price_adj_custom', true );
						$multiplier_custom_value = $price_adj_custom;
						
						error_log( '[CUSTOM] price multiplier = ' . $multiplier_custom_value);
			
						$regular_price = ( (float)$service_credit * (float)$price_adj_custom );
						$regular_price_txt = sprintf("%01.2f", $regular_price);
						
						if ( $woocommerce ) {
							
							// only update if price is different
							if ( $price != $regular_price_txt ) {
								update_post_meta( $product_id, '_suwp_service_credit', $service_credit );
								update_post_meta( $product_id, '_regular_price', $regular_price_txt );
								update_post_meta( $product_id, '_price', $regular_price_txt );
								$regular_price_new = $regular_price_txt;
							}
							
						}
						
						error_log( '[CUSTOM] NEW Product regular price = ' . $regular_price_new);
						
						break;
					case 'global':
						
						// use the settings found in Plugin Options
						$credit = (float)$service_credit;
						$more_equal = (float)$options['suwp_price_range_01']; // more than or equal to
						$more_equal_mult = (float)$options['suwp_price_adj_01']; // more than or equal to multiplier
						$less_equal = (float)$options['suwp_price_range_02']; // less than or equal to
						$less_equal_mult = (float)$options['suwp_price_adj_02']; // less than or equal to multiplier
						$default_mult = (float)$options['suwp_price_adj_default']; // default multiplier
						
						if ( $woocommerce ) {
							
							// check if the global option is enabled
							// 1 = Enabled, '' = disabled
							if ( $options['suwp_price_enabled_01'] === '1' ) {
								
								$multiplier_global_enabled = 'enabled';
								
								if ($credit >= $more_equal) {
									
									$multiplier_global_value = $more_equal_mult;
									
									$regular_price = ( (float)$service_credit * (float)$more_equal_mult );
									$regular_price_txt = sprintf("%01.2f", $regular_price);
											
									// only update if price is different
									if ( $price != $regular_price_txt ) {
										update_post_meta( $product_id, '_suwp_service_credit', $service_credit );
										update_post_meta( $product_id, '_regular_price', $regular_price_txt );
										update_post_meta( $product_id, '_price', $regular_price_txt );
										$regular_price_new = $regular_price_txt;
									}
									
								} elseif ($credit <= $less_equal) {
									
									$multiplier_global_value = $less_equal_mult;
									
									$regular_price = ( (float)$service_credit * (float)$less_equal_mult );
									$regular_price_txt = sprintf("%01.2f", $regular_price);
											
									// only update if price is different
									if ( $price != $regular_price_txt ) {
										update_post_meta( $product_id, '_suwp_service_credit', $service_credit );
										update_post_meta( $product_id, '_regular_price', $regular_price_txt );
										update_post_meta( $product_id, '_price', $regular_price_txt );
										$regular_price_new = $regular_price_txt;
									}
									
								} else {
									
									$multiplier_global_value = $default_mult;
									
									// default multiplier
									$regular_price = ( (float)$service_credit * (float)$default_mult );
									$regular_price_txt = sprintf("%01.2f", $regular_price);
											
									// only update if price is different
									if ( $price != $regular_price_txt ) {
										update_post_meta( $product_id, '_suwp_service_credit', $service_credit );
										update_post_meta( $product_id, '_regular_price', $regular_price_txt );
										update_post_meta( $product_id, '_price', $regular_price_txt );
										$regular_price_new = $regular_price_txt;
									}
									
								} // if ($credit >= $more_equal)
								
							} // if ( $options['suwp_price_enabled_01'] === '1' )
							
							error_log( '[GLOBAL] automatic price adjustment setting = ' . $multiplier_global_enabled);
							error_log( '[GLOBAL] price multiplier = ' . $multiplier_global_value);
							error_log( '[GLOBAL] NEW Product regular price = ' . $regular_price_new);
							
						} // if ( $woocommerce )
					
				} // switch( $price_adj )
			
			} else {
				
				// new import, simply set the Product service credit value (from Supplier)
				update_post_meta( $product_id, '_suwp_service_credit', $service_credit );
				update_post_meta( $product_id, '_regular_price', $service_credit );
				update_post_meta( $product_id, '_price', $service_credit );
				
			}// if ( $flag )
						
		} catch( Exception $e ) {
			
			// a php error occurred
			error_log('----- ERROR - ADDING OR CHECKING FOR PRODUCT TO THE DATABASE ----- ');
			error_log(print_r($e,true));
		}
		
		return $product_array;
		
	}
	
	
	public function suwp_change_price_by_type( $product_id, $multiply_price_by, $price_type ) {
		$the_price = get_post_meta( $product_id, '_' . $price_type, true );
		$the_price *= $multiply_price_by;
		update_post_meta( $product_id, '_' . $price_type, $the_price );
	}
	
	public function suwp_change_price_all_types( $product_id, $multiply_price_by ) {
		$this->suwp_change_price_by_type( $product_id, $multiply_price_by, 'price' );
		$this->suwp_change_price_by_type( $product_id, $multiply_price_by, 'sale_price' );
		$this->suwp_change_price_by_type( $product_id, $multiply_price_by, 'regular_price' );
	}
	
	/*
	 * 'suwp_change_product_price' is main function you should call to change product's price (NOT TESTED)
	 */
	function suwp_change_product_price( $product_id, $multiply_price_by ) {
		$this->suwp_change_price_all_types( $product_id, $multiply_price_by );	
		$product = wc_get_product( $product_id ); // Handling variable products
		if ( $product->is_type( 'variable' ) ) {
			$variations = $product->get_available_variations();
			foreach ( $variations as $variation ) {
				$this->suwp_change_price_all_types( $variation['variation_id'], $multiply_price_by );
			}
		}
	}
	
	public function suwp_add_product_to_database( $product_data ) {
	
		$suwp_api_provider = $product_data['apiproviderid'];
		$suwp_api_service_id = $product_data['api'];
		$suwp_serial_length = 15;
		
		$user_id = get_current_user_id();
		$category = get_term_by( 'slug', 'suwp_service', 'product_cat' );
		$cat_id = $category->term_id;
		
		$regular_price_txt = sprintf("%01.2f", $product_data['credit']);
		
		// None or Required
		
		$post_id = wp_insert_post( array(
			'post_author' => $user_id,
			'post_title' => $product_data['name'],
			'post_content' => $product_data['info'],
			'post_status' => 'imported', // draft, publish
			'post_type' => "product",
		) );
		
		wp_set_object_terms( $post_id, $cat_id, 'product_cat' );
		wp_set_object_terms( $post_id, 'simple', 'product_type' );
		update_post_meta( $post_id, '_visibility', 'visible' );
		update_post_meta( $post_id, '_stock_status', 'instock');
		update_post_meta( $post_id, 'total_sales', '0' );
		update_post_meta( $post_id, '_downloadable', 'no' );
		update_post_meta( $post_id, '_virtual', 'yes' );
		update_post_meta( $post_id, '_regular_price', $regular_price_txt );
		update_post_meta( $post_id, '_price', $regular_price_txt );
		update_post_meta( $post_id, '_sale_price', '' );
		update_post_meta( $post_id, '_purchase_note', '' );
		update_post_meta( $post_id, '_featured', 'no' );
		update_post_meta( $post_id, '_weight', '' );
		update_post_meta( $post_id, '_length', '' );
		update_post_meta( $post_id, '_width', '' );
		update_post_meta( $post_id, '_height', '' );
		update_post_meta( $post_id, '_sku', '' );
		update_post_meta( $post_id, '_product_attributes', array() );
		update_post_meta( $post_id, '_sale_price_dates_from', '' );
		update_post_meta( $post_id, '_sale_price_dates_to', '' );
		update_post_meta( $post_id, '_price', '' );
		update_post_meta( $post_id, '_sold_individually', '' );
		update_post_meta( $post_id, '_manage_stock', 'no' );
		update_post_meta( $post_id, '_backorders', 'no' );
		update_post_meta( $post_id, '_stock', '' );
		update_post_meta( $post_id, '_suwp_serial_length', $suwp_serial_length );
		update_post_meta( $post_id, '_suwp_api_provider', $suwp_api_provider );
		update_post_meta( $post_id, '_suwp_api_service_id', $suwp_api_service_id );
		update_post_meta( $post_id, '_suwp_process_time', $product_data['time'] );
		update_post_meta( $post_id, '_suwp_service_credit', $product_data['credit'] );
		update_post_meta( $post_id, '_suwp_price_group_name', $product_data['groupname'] );
		update_post_meta( $post_id, '_suwp_service_notes', $product_data['info'] );
		update_post_meta( $post_id, '_suwp_is_network', $product_data['provider'] );
		update_post_meta( $post_id, '_suwp_is_model', $product_data['mobile'] );
		update_post_meta( $post_id, '_suwp_is_pin', $product_data['pin'] );
		update_post_meta( $post_id, '_suwp_is_kbh', $product_data['kbh'] );
		update_post_meta( $post_id, '_suwp_is_mep', $product_data['mep'] );
		update_post_meta( $post_id, '_suwp_is_rm_type', $product_data['type'] );
		update_post_meta( $post_id, '_suwp_is_reference', $product_data['reference'] );
		update_post_meta( $post_id, '_suwp_online_status', 'yes' );
	
		return $post_id;
		
	}
	
	// sample Provider entry
	public function suwp_add_stockunlocks_to_database() {
	
		static $suwp_default_created;
		
		if ( $suwp_default_created === null ) {
				
			$user_id = get_current_user_id();
			
			$post_id = wp_insert_post( array(
				'post_author' => $user_id,
				'post_title' => 'The Real 2',
				'post_content' => '',
				'post_status' => 'publish', // draft, publish
				'post_type' => "suwp_apisource",
			) );
			
			update_post_meta( $post_id, 'suwp_activeflag', '1' );
			update_post_meta( $post_id, 'suwp_sitename', 'StockUnlocks');
			update_post_meta( $post_id, 'suwp_url', 'https://reseller.stockunlocks.com/' );
			update_post_meta( $post_id, 'suwp_username', 'Your assigned username' );
			update_post_meta( $post_id, 'suwp_apikey', 'XXX-XXX-XXX-XXX-XXX-XXX-XXX-XXX' );
			update_post_meta( $post_id, 'suwp_apinotes', 'Your Own Mobile Unlocking Website. Please visit https://reseller.stockunlocks.com/singup.html for account creation.' );
			
			$suwp_default_created = $post_id;
				
		}
		
		return $suwp_default_created;
		
	}
	
	// imports new services from our import admin page
	// >>> this function is a form handler and expect services data in the $_POST scope
	public function suwp_import_services() {
		
		error_log( '' );
		error_log( '' );
		error_log('>>>>> ----- START IMPORT SUBMISSION ----- <<<<< ');
		error_log( '' );
		error_log( '' );
		
		$retrieved_posts = get_option('suwp_retrieved_posts');
		error_log('TOTAL = ' . count( $retrieved_posts ) . ' >>>>> ----- RETRIEVED: suwp_retrieved_posts ALL ----- <<<<<' );
		
		// setup our return array
		$result = array(
			'status' => 0,
			'message' => 'Could not import services. ',
			'error' => '',
			'errors' => array(),
		);
		
		try {
			
			// get the provider id to import to
			$api_provider_id = (isset($_POST['suwp_selected_api_provider_id'])) ? (int)$_POST['suwp_selected_api_provider_id'] : 0;
			
			// check api_provider_id contents
			if ( !$api_provider_id ) {
					
				error_log('----- ERROR - NO api_provider_id DETECTED IN IMPORT SUBMISSION, SET TO ZERO ----- ');
	
				$api_provider_id = 0;
				
			} else {
				
				error_log('----- SUCCESS - api_provider_id DETECTED IN IMPORT SUBMISSION >>> ' . $api_provider_id . ' ----- ');
	
			}
			
			$checked_services = (isset($_POST['suwp_selected_service_ids'])) ? (array)$_POST['suwp_selected_service_ids'] : array();
			
			// get the selected services rows to import
			$selected_rows = explode(',', $checked_services[0]);

			$row_count = count($selected_rows);
			
			error_log('----- NUMBER OF ROWS RETURNED FROM IMPORT SUBMISSION >>> ' . $row_count . ' ----- ');
	
			$product_array = array();
						
			// setup a variable for counting the added and modified services
			$added_count = 0;
			$updated_count = 0;
			
			if ( is_array( $selected_rows ) ) {
	
				foreach ( $selected_rows as $api ) {
					
					// 'serviceid' is included when 'all' is selected
					// do a test for is_numeric to exclude it.
					if ( is_numeric( $api ) ) {
						
						$id = intval($api);
						
						error_log('');
						error_log($retrieved_posts[$id]['servicename'] . ' : ----- PROCESSING API ROW DURING IMPORT SUBMISSION >>> ' . $id . ' ----- ');
						
						// build our service data 
						$service_data = array(
							'apiproviderid' => $api_provider_id,
							'api' => $id,
							'name' => $retrieved_posts[$id]['servicename'],
							'time' => $retrieved_posts[$id]['time'],
							'credit' => $retrieved_posts[$id]['credit'],
							'groupname' => $retrieved_posts[$id]['groupname'],
							'info' => $retrieved_posts[$id]['info'],
							'network' => $retrieved_posts[$id]['network'],
							'mobile' => $retrieved_posts[$id]['mobile'],
							'provider' => $retrieved_posts[$id]['provider'],
							'pin' => $retrieved_posts[$id]['pin'],
							'kbh' => $retrieved_posts[$id]['kbh'],
							'mep' => $retrieved_posts[$id]['mep'],
							'prd' => $retrieved_posts[$id]['prd'],
							'type' => $retrieved_posts[$id]['type'],
							'locks' => $retrieved_posts[$id]['locks'],
							'reference' => $retrieved_posts[$id]['reference'],
						);
						
						// add the product to the database
						$product_array = $this->suwp_save_service( $service_data );
						
						// flag: 0 means added, 1 means updated
						$product_id = $product_array['product_id'];
						
						// IF product was created or updated successfully
						if ( $product_id ) {
						
							// updated our added count
							error_log('----- OBTAINED PRODUCT ID DURING IMPORT SUBMISSION >>> ' . $product_id . ' ----- ');
							
							$flag = $product_array['flag'];
							
							if ( $flag ) {
								// updated existing product
								$updated_count++;
							} else {
								// added a new product
								$added_count++;
							}
							
						} else {
							
							error_log('----- ERROR - NO PRODUCT ID OBTAINED DURING IMPORT SUBMISSION, PRODUCT ID IS >>> ' . $product_id . ' ----- ');
							
						} // if ( $product_id ) {
						
					} else {
						
						error_log('----- SKIPPING ROW DURING IMPORT SUBMISSION BECAUSE ITS VALUE IS >>> ' . $api . ', NOT NUMERIC ----- ');
	
					} // if ( is_numeric( $id ) && !empty( $id ) ) {
				}
				
				// IF no products were actually added or updated...
				if( $added_count == 0 && $updated_count == 0):
				
					error_log('----- ERROR - NO SERVICES IMPORTED OR UPDATED INTO TO THE DATABASE >>> MODIFY wp-config.php or php.ini ----- ');
				
					// return error message
					$result['error'] = 'No services were imported or updated. Please select fewer services or modify memory settings in wp-config.php or php.ini';
				
				else:
				
					error_log('----- SUCCESS - ADDED OR UPDATED SERVICES IN THE DATABASE TOTAL >>> ADDED = ' . $added_count . ', UPDATED = ' . $updated_count . ' ----- ');
				
					// IF products were added...
					// return success!
					$result = array(
						'status' => 1,
						'message' => 'SUCCESS - Service(s) : ' . $added_count .' imported, ' . $updated_count . ' updated.',
						'error' => '',
						'errors' => array(),
					);
				
				endif;
			
			} // if ( is_array( $selected_rows ) ) 
		
		} catch( Exception $e ) {
			
			// php error
			error_log('----- PHP ERROR: suwp_import_services() ----- ');
			error_log(print_r($e,true));
			
		}
		
		error_log( '' );
		error_log( '' );
		error_log( '>>>>> ----- END IMPORT SUBMISSION ----- <<<<<' );
		error_log( '' );
		error_log( '' );
		
		update_option('suwp_retrieved_posts', array() );
		
		// find the log and send it
		
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		// return the result as json
		$plugin_public->suwp_exec_return_json( $result );
		
	}
	
	// imports new services from our import admin page
	// >>> this function is a form handler and expect services data in the $_POST scope
	public function suwp_import_services_legacy() {
		
		error_log( '' );
		error_log( '' );
		error_log('>>>>> ----- START IMPORT SUBMISSION ----- <<<<< ');
		error_log( '' );
		error_log( '' );
		
		$retrieved_posts = get_option('suwp_retrieved_posts');
		error_log('TOTAL = ' . count( $retrieved_posts ) . ' >>>>> ----- RETRIEVE: suwp_retrieved_posts ALL ----- <<<<<  = ' . print_r( $retrieved_posts, true) );
		
		// setup our return array
		$result = array(
			'status' => 0,
			'message' => 'Could not import services. ',
			'error' => '',
			'errors' => array(),
		);
		
		try {
			
			// get the assignment values
			$api_column = (isset($_POST['suwp_api_column'])) ? (int)$_POST['suwp_api_column'] : 0;
			$name_column = (isset($_POST['suwp_name_column'])) ? (int)$_POST['suwp_name_column'] : 0;
			$time_column = (isset($_POST['suwp_time_column'])) ? (int)$_POST['suwp_time_column'] : 0;
			$credit_column = (isset($_POST['suwp_credit_column'])) ? (int)$_POST['suwp_credit_column'] : 0;
			$groupname_column = (isset($_POST['suwp_groupname_column'])) ? (int)$_POST['suwp_groupname_column'] : 0;
			$info_column = (isset($_POST['suwp_info_column'])) ? (int)$_POST['suwp_info_column'] : 0;
			$network_column = (isset($_POST['suwp_network_column'])) ? (int)$_POST['suwp_network_column'] : 0;
			$mobile_column = (isset($_POST['suwp_mobile_column'])) ? (int)$_POST['suwp_mobile_column'] : 0;
			$provider_column = (isset($_POST['suwp_provider_column'])) ? (int)$_POST['suwp_provider_column'] : 0;
			$pin_column = (isset($_POST['suwp_pin_column'])) ? (int)$_POST['suwp_pin_column'] : 0;
			$kbh_column = (isset($_POST['suwp_kbh_column'])) ? (int)$_POST['suwp_kbh_column'] : 0;
			$mep_column = (isset($_POST['suwp_mep_column'])) ? (int)$_POST['suwp_mep_column'] : 0;
			$prd_column = (isset($_POST['suwp_prd_column'])) ? (int)$_POST['suwp_prd_column'] : 0;
			$type_column = (isset($_POST['suwp_type_column'])) ? (int)$_POST['suwp_type_column'] : 0;
			$locks_column = (isset($_POST['suwp_locks_column'])) ? (int)$_POST['suwp_locks_column'] : 0;
			$reference_column = (isset($_POST['suwp_reference_column'])) ? (int)$_POST['suwp_reference_column'] : 0;
			
			if ( ! $api_column ) {
				$api_column = 0;
			}
			
			if ( ! $name_column ) {
				$name_column = 0;
			}
			
			if ( ! $time_column ) {
				$time_column = 0;
			}
			
			if ( ! $credit_column ) {
				$credit_column = 0;
			}
			
			if ( ! $groupname_column ) {
				$groupname_column = 0;
			}
			
			if ( ! $info_column ) {
				$info_column = 0;
			}
			
			if ( ! $network_column ) {
				$network_column = 0;
			}
			
			if ( ! $mobile_column ) {
				$mobile_column = 0;
			}
			
			if ( ! $provider_column ) {
				$provider_column = 0;
			}
			
			if ( ! $pin_column ) {
				$pin_column = 0;
			}
			
			if ( ! $kbh_column ) {
				$kbh_column = 0;
			}
			
			if ( ! $mep_column ) {
				$mep_column = 0;
			}
			
			if ( ! $prd_column ) {
				$prd_column = 0;
			}
			
			if ( ! $type_column ) {
				$type_column = 0;
			}
			
			if ( ! $locks_column ) {
				$locks_column = 0;
			}
			
			if ( ! $reference_column ) {
				$reference_column = 0;
			}
			
			// get the provider id to import to
			$api_provider_id = (isset($_POST['suwp_selected_api_provider_id'])) ? (int)$_POST['suwp_selected_api_provider_id'] : 0;
			
			// check api_provider_id contents
			if ( !$api_provider_id ) {
					
				error_log('----- ERROR - NO api_provider_id DETECTED IN IMPORT SUBMISSION, SET TO ZERO ----- ');
	
				$api_provider_id = 0;
				
			} else {
				
				error_log('----- SUCCESS - api_provider_id DETECTED IN IMPORT SUBMISSION >>> ' . $api_provider_id . ' ----- ');
	
			}
			
			// get the selected services rows to import
			$selected_rows = (isset($_POST['suwp_import_rows'])) ? (array)$_POST['suwp_import_rows'] : array();
			
			$row_count = count($selected_rows);
			
			error_log('----- NUMBER OF ROWS RETURNED FROM IMPORT SUBMISSION >>> ' . $row_count . ' ----- ');
	
			$product_array = array();
						
			// setup a variable for counting the added and modified services
			$added_count = 0;
			$updated_count = 0;
			
			if ( is_array( $selected_rows ) ) {
	
				foreach ( $selected_rows as $id ) {
					
					// 'serviceid' is included when 'all' is selected
					// do a test for is_numeric to exclude it.
					// not including the first row, bec it's headers
					if ( is_numeric( $id ) ) {
						
						error_log('');
						error_log('----- PROCESSING API ROW DURING IMPORT SUBMISSION >>> ' . $id . ' ----- ');
						
						// build our service data 
						$service_data = array(
							'apiproviderid' => $api_provider_id,
							'api' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $api_column] ),
							'name' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $name_column] ),
							'time' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $time_column] ),
							'credit' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $credit_column] ),
							'groupname' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $groupname_column] ),
							'info' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $info_column] ),
							'network' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $network_column] ),
							'mobile' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $mobile_column] ),
							'provider' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $provider_column] ),
							'pin' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $pin_column] ),
							'kbh' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $kbh_column] ),
							'mep' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $mep_column] ),
							'prd' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $prd_column] ),
							'type' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $type_column] ),
							'locks' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $locks_column] ),
							'reference' => sanitize_text_field( (string)$_POST['suwp_'. $id .'_'. $reference_column] ),
						);
						
						// add the product to the database
						$product_array = $this->suwp_save_service( $service_data );
						
						// flag: 0 means added, 1 means updated
						$product_id = $product_array['product_id'];
						
						// IF product was created or updated successfully
						if ( $product_id ) {
						
							// updated our added count
							error_log('----- OBTAINED PRODUCT ID DURING IMPORT SUBMISSION >>> ' . $product_id . ' ----- ');
							
							$flag = $product_array['flag'];
							
							if ( $flag ) {
								// updated existing product
								$updated_count++;
							} else {
								// added a new product
								$added_count++;
							}
							
						} else {
							
							error_log('----- ERROR - NO PRODUCT ID OBTAINED DURING IMPORT SUBMISSION, PRODUCT ID IS >>> ' . $product_id . ' ----- ');
							
						} // if ( $product_id ) {
						
					} else {
						
						error_log('----- SKIPPING ROW DURING IMPORT SUBMISSION BECAUSE ITS VALUE IS >>> ' . $id . ', NOT NUMERIC ----- ');
	
					} // if ( is_numeric( $id ) && !empty( $id ) ) {
				}
				
				// IF no products were actually added or updated...
				if( $added_count == 0 && $updated_count == 0):
				
					error_log('----- ERROR - NO SERVICES IMPORTED OR UPDATED INTO TO THE DATABASE >>> MODIFY wp-config.php or php.ini ----- ');
				
					// return error message
					$result['error'] = 'No services were imported or updated. Please select fewer services or modify memory settings in wp-config.php or php.ini';
				
				else:
				
					error_log('----- SUCCESS - ADDED OR UPDATED SERVICES IN THE DATABASE TOTAL >>> ADDED = ' . $added_count . ', UPDATED = ' . $updated_count . ' ----- ');
				
					// IF products were added...
					// return success!
					$result = array(
						'status' => 1,
						'message' => 'SUCCESS - Service(s) : ' . $added_count .' imported, ' . $updated_count . ' updated.',
						'error' => '',
						'errors' => array(),
					);
				
				endif;
			
			} // if ( is_array( $selected_rows ) ) 
		
		} catch( Exception $e ) {
			
			// php error
			error_log('----- PHP ERROR: suwp_import_services() ----- ');
			error_log(print_r($e,true));
			
		}
		
		error_log( '' );
		error_log( '' );
		error_log( '>>>>> ----- END IMPORT SUBMISSION ----- <<<<<' );
		error_log( '' );
		error_log( '' );
		
		// find the log and send it
		
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		// return the result as json
		$plugin_public->suwp_exec_return_json( $result );
		
	}

	// this function retrieves services data from the remote server in the form of a php array
	// it then returns that array in a json formatted object
	// this function is an ajax post form handler
	// expects: $_POST['suwp_import_provider_list_id']
	public function suwp_parse_import_api() {
		
		error_log( '' );
		error_log( '' );
		error_log('>>>>> ----- START RETRIEVE ... ----- <<<<< ');
		error_log( '' );
		error_log( '' );
		
		// setup our return array
		$result = array(
			'status' => 0,
			'provider_id' => '',
			'message' => 'Could not import remote services. ',
			'error' => '',
			'data1' => array(),
			'data2' => array(),
		);
		
		try {
		
			// get the provider id from $_POST['suwp_import_provider_list_id']
			$provider_id = (isset($_POST['suwp_import_provider_list_id'])) ? esc_attr( $_POST['suwp_import_provider_list_id'] ) : 0;
			
			error_log( '' );
			error_log( '' );
			error_log('>>>>> ----- RETRIEVE: PROVIDER ID ----- <<<<<  = ' . $provider_id);
			error_log( '' );
			error_log( '' );
			
			$api_id = (int)sanitize_text_field($provider_id);
			
			if ( ! $api_id ) {
				$api_id = 0;
			}
			
			$reply = $this->suwp_api_ui_action( $api_id, $api_action_id = 1 );
			
			// $reply[0] = $reply_part
			// $reply[1] = $reply_all
			
			// if there is a reply
			if( !empty($reply) ):
				
				update_option( 'suwp_retrieved_posts', $reply[1] );
				
				// setup our return array
				$result = array(
					'status'=> 1,
					'provider_id' => $provider_id,
					'message'=> 'Imported remote services successfully.',
					'error'=> '',
					'data1'=> $reply[0],
					'data2'=> $reply[1],
				);
				
			else:
			
				// return an error message if we could not retrieve the file
				$result['error']='Failed connection to the remote server or no assigned services available. ';
				$result['data1']=$reply;
				
			endif;
			
		} catch( Exception $e ) {
			
			// php error
				
			error_log( '' );
			error_log( '' );
			error_log('>>>>> ----- RETRIEVE: PHP ERROR ----- <<<<<  = ' . print_r( $e, true ) );
			error_log( '' );
			error_log( '' );
		
		}
		
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		// return the result as json
		$plugin_public->suwp_exec_return_json( $result );
		
	}
	
	// generates a .csv file of services data
	// expects $_GET['provider_id'] to be set in the URL
	public function suwp_download_services_csv() {
		
		// reset flag to remove the "No Unlocking Products to export." notice
		$_SESSION['suwp_services_export_flag'] = '1';
	
		// get the provider id from the URL scope
		$provider_id = ( isset($_GET['provider_id']) ) ? (int)$_GET['provider_id'] : 0;
		
		// setup our return data
		$csv = '';
		
		// get the provider object
		$provider = get_post( $provider_id );
		
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		// get the provider's services or get all services if no provider id is given
		$services = $plugin_public->suwp_exec_get_provider_services( $provider_id );
		
		// IF we have confirmed services
		if( $services !== false ):
		
			// get the current date
			$now = new DateTime();
			
			// setup a unique filename for the generated export file
			$fn1 = 'stockunlocks-export-provider_id-'. $provider_id .'-date-'. $now->format('Ymd'). '.csv';
			$fn2 = WP_PLUGIN_DIR .'/stockunlocks/exports/'.$fn1;
			
			// open new file in write mode
			$fp = fopen($fn2, 'w');
			
			$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
			
			// get the first services' data
			$service_data = $plugin_public->suwp_exec_get_service_data( $services[0] );
			
			// remove the subscriptions and name column from the data
			// unset($service_data['subscriptions']);
			// unset($service_data['name']);
			
			// build our csv headers array from $service_data's data keys
			$csv_headers = array();
			foreach( $service_data as $key => $value ):
				array_push($csv_headers, $key);
			endforeach;
			
			// append $csv_headers to our csv file
			fputcsv($fp, $csv_headers);
		
			// loop over all our services
			foreach( $services as &$service_id ):
					
				$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
				
				// get the service data of the current service
				$service_data = $plugin_public->suwp_exec_get_service_data( $service_id );
			
				// remove the subscriptions and name columns from the data
				// unset($service_data['subscriptions']);
				// unset($service_data['name']);
				
				// append this services' data to our csv file
				fputcsv($fp, $service_data);
			
			endforeach;
			
			// read open our new file is read mode
			$fp = fopen($fn2, 'r');
			// read our new csv file and store it's contents in $fc
			$fc = fread($fp, filesize($fn2) );
			// close our open file pointer
			fclose($fp);
		
			// setup file headers
			header("Content-type: application/csv");
			header("Content-Disposition: attachment; filename=".$fn1);
			// echo the contents of our file and return it to the browser
			echo($fc);
			// exit php processes 
			exit;
			
		else:
			
			wp_redirect(admin_url('/admin.php?page=suwp_dashboard_admin_page'), 302);
			
			// set the flag to display the "No Unlocking Products to export." notice
			$_SESSION['suwp_services_export_flag'] = '0';
			
			// stop all other processing 
			exit;
		
		endif;
		
		// return false if we were unable to download our csv
		return false;
		
	}
	
	/**
	* Checks the current version of wordpress and displays
	* message in the plugin page if the version is untested
	*/
	public function suwp_check_wp_version() {
		
		global $pagenow;
		
		
		if ( $pagenow == 'plugins.php' && is_plugin_active('stockunlocks/stockunlocks.php') ):
		
			// get the wp version
			$wp_version = get_bloginfo('version');
			
			// tested vesions
			// these are the versions we've tested our plugin in
			$tested_versions = array(
				'4.2.0',
				'4.2.1',
				'4.2.2',
				'4.2.3',
				'4.2.4',
				'4.2.5',
				'4.2.6',
				'4.7',
				'4.7.1',
			);
			
			$tested_range = array(4.0,4.6);
			
			// IF the current wp version is  in our tested versions...
			if( (float)$wp_version >= (float)$tested_range[0] && (float)$wp_version <= (float)$tested_range[1] ):
			
				// we're good!
			
			else:
				
				// get notice html
				// $notice = suwp_exec_get_admin_notice('StockUnlocks has not been tested in your version of WordPress. It may still work though...','error');
				$notice = do_action('suwp_exec_get_admin_notice','StockUnlocks has not been tested in your version of WordPress. It may still work though...','error');
				
				// echo the notice html
				// echo( $notice );
				
			endif;
		
		endif;
		
	}
	
	/**
	* Returns html formatted
	* for WP admin notices
	*/
	public function suwp_exec_get_admin_notice( $message, $class ) {
		
		// setup our return variable
		$output = '';
		
		try {
			
			// create output html
			$output = '
			 <div class="'. $class .'">
				<p>'. $message .'</p>
			</div>
			';
			
		} catch( Exception $e ) {
			
			// php error
			
		}
		
		// return output
		return $output;
		
	}
	
	public function suwp_no_services_admin_notice() {
	
		$screen = get_current_screen();
		
		if ( $screen->id == 'toplevel_page_suwp_dashboard_admin_page' ) {
			
			if (isset( $_SESSION['suwp_services_export_flag']) ) {
				
				if ( $_SESSION['suwp_services_export_flag'] == '0') {
					$notice = $this->suwp_exec_get_admin_notice('No Unlocking Products to export.','notice notice-warning is-dismissible');
					echo $notice;
					
				}
			}
		}
	}
	
	public function suwp_troubleshooting_admin_notice() {
		
		$screen = get_current_screen();
		$redirect_url = '';
		
		if ( $screen->id == 'stockunlocks_page_suwp_importservices_admin_page' ) {
			$redirect_url = '. <a href="'. admin_url('/admin.php?page=suwp_options_admin_page') .'">Disable here</a>';
		}
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$troubleshoot_items = $options['suwp_manage_troubleshoot_run_id'];
		// parent_base ; parent_file ; suwp_dashboard_admin_page ; suwp_importservices_admin_page
		if ( ( $screen->id == 'stockunlocks_page_suwp_importservices_admin_page' || $screen->id == 'stockunlocks_page_suwp_options_admin_page' ) && $troubleshoot_items > 0 ) {
				$notice = $this->suwp_exec_get_admin_notice('Troubleshooting Option is enabled. Now limiting Service Imports to : ' . $troubleshoot_items . '' . $redirect_url,'notice notice-error ');
				echo $notice;
		}
	}
	
	public function suwp_cron_disabled_admin_notice() {
		
		$screen = get_current_screen();
		$redirect_url = '';
		
		if ( $screen->id !== 'stockunlocks_page_suwp_options_admin_page' ) {
			$redirect_url = '<a href="'. admin_url('/admin.php?page=suwp_options_admin_page') .'">Enable here</a>';
		}
		
		// get the default values for our options
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );

		$options = $plugin_public->suwp_exec_get_current_options();
		
		$cron_run = $options['suwp_manage_cron_run_id'];
		
		if ( ( $screen->parent_file == 'suwp_dashboard_admin_page' ) && $cron_run == '') {
				$notice = $this->suwp_exec_get_admin_notice('Cron is disabled for the StockUnlocks plugin. No orders are being processed with your Provider(s). Automated emails are not being sent out. '. $redirect_url,'notice notice-error ');
				echo $notice;
		}
	}
	
	/**
	* Registers all our plugin options
	* to be used throughout 
	*/
	public function suwp_register_options() {
		
		// make sure that we can use session variables
		if (!session_id()) {
			session_start();
		}
		
		// get plugin options settings
		$plugin_public = new Stock_Unlocks_Public( 'stockunlocks', STOCKUNLOCKS_VERSION );
		
		$options = $plugin_public->suwp_exec_get_options_settings();
		
		// loop over settings
		foreach( $options['settings'] as $setting ):
		
			// register this setting
			register_setting($options['group'], $setting);
		
		endforeach;
		
		$this->suwp_notice_ignore();
	}
	
	/* This is the action that allows
	* the user to dismiss the banner
	* it basically sets a tag to their 
	* user meta data
	*/
	public function suwp_notice_ignore() {
		
		//Get the global user
		global $current_user;
		$user_id = $current_user->ID;
		
		/* If user clicks to ignore the notice, 
		* add that to their user meta 
		* the banner then checks whether this tag
		* exists already or not.
		* See here: http://codex.wordpress.org/Function_Reference/add_user_meta
		*/
		// standard dismissable banner for plugin page after plugin activation
		if ( isset($_GET['suwp_notice_ignore']) && '0' == $_GET['suwp_notice_ignore'] ) {
		   add_user_meta($user_id, 'suwp_activation_ignore_notice', 'true', true);
		}
		
		// remote dismissable banner for plugin page only
		$dismiss_1_key = '';
		$extract = get_option('suwp_author_info');
		
		if( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$dismiss_1_key = $extract->dismiss_1_key;
				
				if ( isset($_GET[$dismiss_1_key]) && '0' == $_GET[$dismiss_1_key] ) {
				   add_user_meta($user_id, $dismiss_1_key, 'true', true);
				}
			}
		}
	}
	
	/* Display a standard notice that can be dismissed 
	* Just leverages WordPress's core function
	* admin_notices
	* See here: http://codex.wordpress.org/Plugin_API/Action_Reference/admin_notices
	*/
	public function suwp_activation_admin_notice() {
		
		$dismiss_1_image = '';
		$extract = get_option('suwp_author_info');
		
		if( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$dismiss_1_image = $extract->dismiss_1_image;
			}
		}
		
		//Get current user
		global $current_user ;
		$user_id = $current_user->ID;
		
		//Get the current page to add the notice to
		global $pagenow;
		
		//Make sure we're on the plugins page.
		if ( $pagenow == 'plugins.php' ) {
			
			// delete_user_meta( $user_id, 'suwp_activation_ignore_notice', 'true' ); // >>>>>>>>>>>>>> REMOVE THIS FOR PRODUCTION
			
			// If the user hasn't already dismissed the alert, 
			// Output the activation banner
			if (!get_user_meta($user_id, 'suwp_activation_ignore_notice')) {
			 
				 include( SUWP_PATH . 'admin/partials/stockunlocks-admin-install-banner.php' );
			 
			}
		}
	}
	
	/* Display a remote notice that can be dismissed 
	* Just leverages WordPress's core function
	* admin_notices
	* See here: http://codex.wordpress.org/Plugin_API/Action_Reference/admin_notices
	*/
	public function suwp_remote_plugin_admin_notice() {
		
		$extract = get_option('suwp_author_info');
		
		if( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$dismiss_1_banner = $extract->dismiss_1_banner;
				$dismiss_1_color = $extract->dismiss_1_color;
				$dismiss_1_key  = $extract->dismiss_1_key;
				$dismiss_1_key_del  = $extract->dismiss_1_key_del;
				$dismiss_1_path = $extract->dismiss_1_path;
				$dismiss_1_image = $extract->dismiss_1_image;
				$dismiss_1_msg  = $extract->dismiss_1_msg;
				$dismiss_1_dashtxt = $extract->dismiss_1_dashtxt;
				$dismiss_1_dashlink = $extract->dismiss_1_dashlink;
				$dismiss_1_dashtarget = $extract->dismiss_1_dashtarget;
				
				//Get current user
				global $current_user ;
				$user_id = $current_user->ID;
				
				//Get the current page to add the notice to
				global $pagenow;
				
				//Make sure we're on the plugins page.
				if ( $pagenow == 'plugins.php' ) {
					
					// delete previously used meta key
					delete_user_meta( $user_id, $dismiss_1_key_del );
					
					// If the user hasn't already dismissed the alert, 
					// Output the activation banner
					if ( !get_user_meta($user_id, $dismiss_1_key ) && $dismiss_1_banner === 'true') {
					 
						 include( SUWP_PATH . $dismiss_1_path );
					}
				}
			}
		}
	}
	
	/* Display a remote notice that can be dismissed 
	* Just leverages WordPress's core function
	* admin_notices
	* See here: http://codex.wordpress.org/Plugin_API/Action_Reference/admin_notices
	*/
	public function suwp_remote_notice_ignore() {
		
		//Get the global user
		global $current_user;
		$user_id = $current_user->ID;
		
		/* If user clicks to ignore the notice, 
		* add that to their user meta 
		* the banner then checks whether this tag
		* exists already or not.
		* See here: http://codex.wordpress.org/Function_Reference/add_user_meta
		*/
		$extract = get_option('suwp_author_info');
		
		if( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				
				$dismiss_0_key  = $extract->dismiss_0_key;
				
				if( isset( $_GET[$dismiss_0_key] ) && '0' == $_GET[$dismiss_0_key] && isset( $_GET['screen_id_url'] ) ) {
					
					$screen_id_url = $_GET['screen_id_url'];
					$admin_url = '';
					
					switch( $screen_id_url ) {
						
						case 'toplevel_page_suwp_dashboard_admin_page':
							$admin_url = '/admin.php?page=suwp_dashboard_admin_page';
							break;
						case 'edit-suwp_apisource':
							$admin_url = '/edit.php?post_type=suwp_apisource';
							break;
						case 'stockunlocks_page_suwp_importservices_admin_page':
							$admin_url = '/admin.php?page=suwp_importservices_admin_page';
							break;
						case 'stockunlocks_page_suwp_options_admin_page':
							$admin_url = '/admin.php?page=suwp_options_admin_page';
							break;
					}
					
					if( isset( $admin_url ) ) {
						
						add_user_meta($user_id, $dismiss_0_key, 'true', true);
						wp_redirect( admin_url( $admin_url ), 302 );
						
						// stop all other processing 
						exit;
					}
					
				} else {
					
					// return false if unable to obtain url variables 
					return false;
				}
			}
		}
	}
	
	public function suwp_remote_admin_notice() {
		
		$parent_file = 'suwp_dashboard_admin_page';
		$extract = get_option('suwp_author_info');
		
		if( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$dismiss_0_banner = $extract->dismiss_0_banner;
				$dismiss_0_color = $extract->dismiss_0_color;
				$dismiss_0_key  = $extract->dismiss_0_key;
				$dismiss_0_key_del  = $extract->dismiss_0_key_del;
				$dismiss_0_path = $extract->dismiss_0_path;
				$dismiss_0_image = $extract->dismiss_0_image;
				$dismiss_0_msg  = $extract->dismiss_0_msg;
				$dismiss_0_dashtxt = $extract->dismiss_0_dashtxt;
				$dismiss_0_dashlink = $extract->dismiss_0_dashlink;
				$dismiss_0_dashtarget = $extract->dismiss_0_dashtarget;
					
			   //Get current user
			   global $current_user ;
			   $user_id = $current_user->ID;
			   
			   //Get the current page to add the notice to
			   global $pagenow;
			   
				if (is_object( get_current_screen() ) ) {
					
					$screen = get_current_screen();
					
					// Make sure we're on pages specific to the StockUnlocks plugin only.
					if ( $screen->parent_file === $parent_file ) {
						
						// delete previously used meta key
						delete_user_meta( $user_id, $dismiss_0_key_del );
						
						// If it's active and if the user hasn't already dismissed the alert,
						// Output the remote banner
						if ( !get_user_meta($user_id, $dismiss_0_key ) && $dismiss_0_banner === 'true') {
						
							include( SUWP_PATH . $dismiss_0_path );
						}
					}
				}
			}
		}
	}
	
	// returns a unique link for closing the remote banner
	public function suwp_get_remote_notice_ignore_link( $ingnore_id = 0 ) {
		
		// $dismiss_0_key = 'suwp_remote_ignore_notice';
		
		$extract = get_option('suwp_author_info');
		
		if( is_object( $extract ) ) {
			if( !isset($extract->error) ) {
				$dismiss_0_key  = $extract->dismiss_0_key;
					
				$link_href = 'admin-ajax.php?action=suwp_remote_notice_ignore&' . $dismiss_0_key . '='. $ingnore_id;
				// return the remove link
				return esc_url($link_href);
			}
		}
		
		return '';
	}
	
	// messages for the admin
	public function suwp_default_admin_footer() {
		
		// >>> $plugin_admin = new Stock_Unlocks_Admin( 'stockunlocks', STOCKUNLOCKS_VERSION );
		$show_notice = false;
		$screen = get_current_screen();
		$extract = get_option('suwp_author_info');
					
		if( is_object( $extract ) ) {
			
			if( isset($extract->error) ) {
				
				if( is_object( get_current_screen() ) ) {
					
					$screen = get_current_screen();
					
					if( $screen->id !== 'stockunlocks_page_suwp_options_admin_page' ) {
							
						if( $screen->parent_file == 'suwp_dashboard_admin_page' ) {
									
							wp_redirect(admin_url('/admin.php?page=suwp_options_admin_page'), 302);
							
							// stop all other processing 
							exit;
						}
						
					} else {
						
						// need to refresh the admin dashboard links to get new 'Providers' link
						// otherwise, clicking on any other StockUnlocks sub_menu will do it
					}
					
					if( $screen->parent_file == 'suwp_dashboard_admin_page' ) {
										
						// >>> $notice = $plugin_admin->suwp_exec_get_admin_notice( $extract->error . ' Please enter the correct values and try again. To use the basic license, simply clear all field values and <strong>Save Changes</strong>. If you do not see the fields, please upgrade the StockUnlocks plugin to the latest version.', 'error notice');
						$notice = $this->suwp_exec_get_admin_notice( $extract->error . ' Please enter the correct values and try again. To use the basic license, simply clear all field values and <strong>Save Changes</strong>. The basic license information will automatically appear.', 'error notice');
						
						// echo the notice html
						echo( $notice );
					}
				}
				
			} else {
			
				$show_notice = true;
			}
			
		} else {
			
			// connection problem
			if (is_object( get_current_screen() ) ) {
				
				$screen = get_current_screen();
			
				if( $screen->id !== 'stockunlocks_page_suwp_options_admin_page' ) {
						
					if( $screen->parent_file == 'suwp_dashboard_admin_page' ) {
								
						wp_redirect(admin_url('/admin.php?page=suwp_options_admin_page'), 302);
						
						// stop all other processing 
						exit;
					}
				}
			
				if( $screen->parent_file == 'suwp_dashboard_admin_page' ) {
									
					// updated notice, error notice, update-nag notice, is-dismissible
					// >>> $notice = $plugin_admin->suwp_exec_get_admin_notice('ERROR- No connection to the license server. Restore the connection and enter a valid license e-mail and key combination and <strong>Save Changes</strong>.', 'error notice');
					$notice = $this->suwp_exec_get_admin_notice('ERROR- No connection to the license server. Restore the connection and enter a valid license e-mail and key combination and <strong>Save Changes</strong>.', 'error notice');
					
					// echo the notice html
					echo( $notice );
				}
			}
		}
		
		if( $show_notice ) {
			
			$latest_version = $extract->version;
			$msg_0_banner = $extract->msg_0_banner;
			$msg_0 = $extract->msg_0;
			$msg_0_class = $extract->msg_0_class;
			$msg_0_link = $extract->msg_0_link;
			$msg_0_link_text = $extract->msg_0_link_text;
			$msg_0_link_target = $extract->msg_0_link_target;
			
			if ($msg_0_banner === 'true') {
				
				if (is_object( get_current_screen() ) ) {
					
					$screen = get_current_screen();
					
					if( $screen->parent_file == 'suwp_dashboard_admin_page' || $screen->id == 'suwp_apisource') {
						
						// 'target="_blank"';
						
						// $redirect_url = '<a href="'. 'https://www.stockunlocks.com/pro' .'">More Info</a>';
						$redirect_url = '<a href="'. $msg_0_link .'" '. $msg_0_link_target . '>' . $msg_0_link_text . '</a>';
						
						// the 'update-nag notice' class messes up the Providers custom post type view
						// simply replace with the 'updated notice' for now.
						if( $screen->id == 'edit-suwp_apisource' || $screen->id == 'suwp_apisource' ) {
							$msg_0_class = 'updated notice';
						}
						
						// updated notice, error notice, update-nag notice, is-dismissible
						// >>> $notice = $plugin_admin->suwp_exec_get_admin_notice($msg_0 . '  ' . $redirect_url, $msg_0_class );
						$notice = $this->suwp_exec_get_admin_notice($msg_0 . '  ' . $redirect_url, $msg_0_class );
						
						// echo the notice html
						echo( $notice );
					}
				}
			}
			
			// upgrade? only for Providers screen
			$msg_3_banner = $extract->msg_3_banner;
			$msg_3 = $extract->msg_3;
			$msg_3_class = $extract->msg_3_class;
			$msg_3_link = $extract->msg_3_link;
			$msg_3_link_text = $extract->msg_3_link_text;
			$msg_3_link_target = $extract->msg_3_link_target;
			
			if ($msg_3_banner === 'true') {
				
				if (is_object( get_current_screen() ) ) {
					
					$screen = get_current_screen();
					
					if( $screen->parent_file == 'suwp_dashboard_admin_page' || $screen->id == 'suwp_apisource') {
						
						// the 'update-nag notice' class messes up the Providers custom post type view
						// simply replace with the 'updated notice' for now.
						if( $screen->id == 'edit-suwp_apisource' || $screen->id == 'suwp_apisource' ) {
							$msg_3_class = 'updated notice';
							
							// 'target="_blank"';
							
							// $redirect_url = '<a href="'. 'https://www.stockunlocks.com/pro' .'">More Info</a>';
							$redirect_url = '<a href="'. $msg_3_link .'" '. $msg_3_link_target . '>' . $msg_3_link_text . '</a>';
							
							// updated notice, error notice, update-nag notice, is-dismissible
							// >>> $notice = $plugin_admin->suwp_exec_get_admin_notice($msg_0 . '  ' . $redirect_url, $msg_0_class );
							$notice = $this->suwp_exec_get_admin_notice($msg_3 . '  ' . $redirect_url, $msg_3_class );
							
							// echo the notice html
							echo( $notice );
						
						}
					}
				}
			}
			
			if( STOCKUNLOCKS_VERSION < $latest_version ) {
				
				if (is_object( get_current_screen() ) ) {
					
					$screen = get_current_screen();
					$redirect_url = '';
					
					if ( $screen->parent_file == 'suwp_dashboard_admin_page' ) {
					
						$redirect_url = '<a href="'. admin_url('/update-core.php') .'">Update Here</a>';
						
						// updated notice, error notice, update-nag notice, is-dismissible
						// >>> $notice = $plugin_admin->suwp_exec_get_admin_notice('The StockUnlocks plugin needs to be upgraded. You are using version: <strong>' . STOCKUNLOCKS_VERSION . '</strong>. The latest version is: <strong>' . $latest_version . '</strong>. ' . $redirect_url,'update-nag notice');
						$notice = $this->suwp_exec_get_admin_notice('The StockUnlocks plugin needs to be udpated. You are using version: <strong>' . STOCKUNLOCKS_VERSION . '</strong>. The latest version is: <strong>' . $latest_version . '</strong>. ' . $redirect_url,'updated notice');
						
						// echo the notice html
						echo( $notice );
					}
				}
			}
		}	
	}
	
	// hint: increases _suwp_qty_sent count by one
	public function suwp_update_qty_sent( $uid ) {
		
		global $wpdb;
		
		// setup our return value
		$return_value = false;
		
		try {
			
			$table_name = $wpdb->prefix . "woocommerce_order_itemmeta";
			
			// get current imei quantity sent count
			$current_count = $wpdb->get_var( 
				$wpdb->prepare( 
					"
						SELECT meta_value 
						FROM $table_name 
						WHERE meta_key = '_suwp_qty_sent' AND order_item_id = %s
					", 
					$uid
				) 
			);
			
			// set new count
			$new_count = (int)$current_count+1;
			
			// update imei quantity sent for this order entry
			$wpdb->query(
				$wpdb->prepare( 
					"
						UPDATE $table_name
						SET meta_value = $new_count 
						WHERE meta_key = '_suwp_qty_sent' AND order_item_id = %s
					", 
					$uid
				) 
			);
			
			$return_value = true;
			
		} catch( Exception $e ) {
			
			// php error
			
		}
		
		return $return_value;
		
	}

	// hint: return the _suwp_qty_sent value; the typical woocommerce function was caching results
	public function suwp_get_qty_sent( $uid ) {
		
		global $wpdb;
		
		// setup our return value
		$return_value = -1;
		
		try {
			
			$table_name = $wpdb->prefix . "woocommerce_order_itemmeta";
			
			// get current imei quantity sent count
			$current_count = $wpdb->get_var( 
				$wpdb->prepare( 
					"
						SELECT meta_value 
						FROM $table_name 
						WHERE meta_key = '_suwp_qty_sent' AND order_item_id = %s
					", 
					$uid
				) 
			);
			
			// set return value
			$return_value = $current_count;
			
		} catch( Exception $e ) {
			
			// php error
			
		}
		
		return $return_value;
		
	}

	// hint: increases _suwp_qty_done count by one
	public function suwp_update_qty_done( $uid ) {
		
		global $wpdb;
		
		// setup our return value
		$return_value = false;
		
		try {
			
			$table_name = $wpdb->prefix . "woocommerce_order_itemmeta";
			
			// get current imei quantity done (replied) count
			$current_count = $wpdb->get_var( 
				$wpdb->prepare( 
					"
						SELECT meta_value 
						FROM $table_name 
						WHERE meta_key = '_suwp_qty_done' AND order_item_id = %s
					", 
					$uid
				) 
			);
			
			// set new count
			$new_count = (int)$current_count+1;
			
			// update imei quantity done (replied) for this order entry
			$wpdb->query(
				$wpdb->prepare( 
					"
						UPDATE $table_name
						SET meta_value = $new_count 
						WHERE meta_key = '_suwp_qty_done' AND order_item_id = %s
					", 
					$uid
				) 
			);
			
			$return_value = true;
			
		} catch( Exception $e ) {
			
			// php error
			
		}
		
		return $return_value;
		
	}
	
	// hint: return the _suwp_qty_done value; the typical function was caching results
	public function suwp_get_qty_done( $uid ) {
		
		global $wpdb;
		
		// setup our return value
		$return_value = -1;
		
		try {
			
			$table_name = $wpdb->prefix . "woocommerce_order_itemmeta";
			
			// get current imei quantity done (replied) count
			$current_count = $wpdb->get_var( 
				$wpdb->prepare( 
					"
						SELECT meta_value 
						FROM $table_name 
						WHERE meta_key = '_suwp_qty_done' AND order_item_id = %s
					", 
					$uid
				) 
			);
			
			// set return value
			$return_value = $current_count;
			
		} catch( Exception $e ) {
			
			// php error
			
		}
		
		return $return_value;
		
	}
	
	public function suwp_search_for_comment_type($id, $array) {
		foreach ($array as $key => $val) {
		   if ($val['comment_type'] === $id) {
			   return $key;
		   }
		}
		return null;
	}

}
